/**
 * Copyright (C) 2010  BonitaSoft S.A.
 * BonitaSoft, 31 rue Gustave Eiffel - 38000 Grenoble
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade;

import java.util.logging.Level;
import java.util.logging.Logger;

import org.ow2.bonita.util.BonitaRuntimeException;
import org.ow2.bonita.util.ExceptionManager;

public class AutoDetectSecurityContext implements BonitaSecurityContext {

  private static final Logger LOG = Logger.getLogger(AutoDetectSecurityContext.class.getName());

  public String getUser() {
    if (LOG.isLoggable(Level.FINEST)) {
      LOG.finest("Using AutoDetectSecurityContext to get the current logged user.");
    }
    String user = getUser(new EJB3SecurityContext());
    if (user == null) {
      user = getUser(new EJB2SecurityContext());
    }
    if (user == null) {
      user = getUser(new StandardSecurityContext());
    }
    if (user == null) {
      if (LOG.isLoggable(Level.FINEST)) {
        LOG.finest("No user found from " + AutoDetectSecurityContext.class);
      }
      String message = ExceptionManager.getInstance().getFullMessage("bsi_ADSC_1");
      throw new BonitaRuntimeException(message);
    }
    return user;
  }
  
  private String getUser(BonitaSecurityContext securityContext) {
    try {
      return securityContext.getUser();
    } catch (Throwable t) {
      if (LOG.isLoggable(Level.FINEST)) {
        LOG.finest("No user found from " + securityContext.getClass());
      }
      return null;
    }
  }
}
