/**
 * Copyright (C) 2010  BonitaSoft S.A.
 * BonitaSoft, 31 rue Gustave Eiffel - 38000 Grenoble
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade;

import java.lang.reflect.Method;
import java.security.Principal;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.ow2.bonita.util.BonitaRuntimeException;
import org.ow2.bonita.util.Misc;

public class EJB3SecurityContext implements BonitaSecurityContext {

  private static final Logger LOG = Logger.getLogger(EJB3SecurityContext.class.getName());
  
  public String getUser() {
    if (LOG.isLoggable(Level.FINEST)) {
      LOG.finest("Looking for user in " + EJB3SecurityContext.class);
    }
    try {
      final Object ejbContext = Misc.lookup("java:comp/EJBContext", null);
      if (LOG.isLoggable(Level.FINEST)) {
        LOG.finest("Looking for user in " + EJB3SecurityContext.class + ", ejbContext=" + ejbContext);
      }
      final Class< ? > ejbContextClass = Class.forName("javax.ejb.EJBContext");
      final Method getCallerPrincipalMethod = ejbContextClass.getMethod("getCallerPrincipal");
      final Principal principal = (Principal) getCallerPrincipalMethod.invoke(ejbContext);
      String user = principal.getName();
      if (LOG.isLoggable(Level.FINEST)) {
        LOG.finest("User found in " + EJB3SecurityContext.class + ": " + user);
      }
      return user;
    } catch (Exception e) {
      if (LOG.isLoggable(Level.FINEST)) {
        LOG.finest("Exception caught while looking for user in " + EJB3SecurityContext.class + ": " + e.getMessage());
      }
      throw new BonitaRuntimeException(e);
    }
  }

}
