/**
 * Copyright (C) 2010  BonitaSoft S.A.
 * BonitaSoft, 31 rue Gustave Eiffel - 38000 Grenoble
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade.ejb;

import java.util.logging.Level;
import java.util.logging.Logger;

import org.ow2.bonita.util.ExceptionManager;
import org.ow2.bonita.util.Misc;

public final class EJB2SecurityOwner {

  private static final InheritableThreadLocal<String> USERS = new InheritableThreadLocal<String>();
  private static final Logger LOG = Logger.getLogger(EJB2SecurityOwner.class.getName());
  
  private EJB2SecurityOwner() { }
  
  public static String getUser() {
    if (LOG.isLoggable(Level.FINEST)) {
      LOG.entering(EJB2SecurityOwner.class.getName(), "getUser");
    }
    final String userId = USERS.get();
    String message = ExceptionManager.getInstance().getFullMessage("bsi_EJBSO_1");    
    Misc.badStateIfNull(userId, message);

    if (LOG.isLoggable(Level.FINEST)) {
      LOG.exiting(EJB2SecurityOwner.class.getName(), "getUser", userId);
    }
    return userId;
  }
  
  static void setUser(String userId) {
    if (LOG.isLoggable(Level.FINEST)) {
      LOG.entering(EJB2SecurityOwner.class.getName(), "setUser", userId);
    }
    String message = ExceptionManager.getInstance().getFullMessage("bsi_EJBSO_2");
    Misc.checkArgsNotNull(userId, message);
    USERS.set(userId);
    if (LOG.isLoggable(Level.FINE)) {
      LOG.fine("User set: " + userId);
    }
    if (LOG.isLoggable(Level.FINEST)) {
      LOG.exiting(EJB2SecurityOwner.class.getName(), "setUser");
    }
  }

}
