/**
 * Copyright (C) 2010  BonitaSoft S.A.
 * BonitaSoft, 31 rue Gustave Eiffel - 38000 Grenoble
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade.ejb;

import java.rmi.RemoteException;
import java.util.Set;

import javax.ejb.SessionBean;
import javax.ejb.SessionContext;

import org.ow2.bonita.facade.IdentityAPI;
import org.ow2.bonita.facade.exception.RoleAlreadyExistsException;
import org.ow2.bonita.facade.exception.RoleNotFoundException;
import org.ow2.bonita.facade.exception.UserAlreadyExistsException;
import org.ow2.bonita.facade.exception.UserNotFoundException;
import org.ow2.bonita.facade.identity.Role;
import org.ow2.bonita.facade.identity.User;
import org.ow2.bonita.facade.impl.StandardAPIAccessorImpl;
import org.ow2.bonita.facade.internal.InternalIdentityAPI;

public class IdentityAPIBean implements InternalIdentityAPI, SessionBean {

  private static final long serialVersionUID = -8220749024878119018L;
  
  protected SessionContext ctx = null;

  protected IdentityAPI identityAPI;

  protected IdentityAPI getAPI() {
    EJB2SecurityOwner.setUser(this.ctx.getCallerPrincipal().getName());
    if (this.identityAPI == null) {
      this.identityAPI = new StandardAPIAccessorImpl().getIdentityAPI();
    }
    return this.identityAPI;
  }

  public void ejbCreate() {

  }

  public void ejbActivate() throws RemoteException {

  }

  public void ejbPassivate() throws RemoteException {

  }

  public void ejbRemove() throws RemoteException {

  }

  public void setSessionContext(final SessionContext arg0) throws RemoteException {
    this.ctx = arg0;
  }

  public Role addRole(String name) throws RemoteException, RoleAlreadyExistsException {
    return getAPI().addRole(name);
  }

  public Role addRole(String name, String label, String description) throws RemoteException, RoleAlreadyExistsException {
    return getAPI().addRole(name, label, description);
  }

  public void addRoleToUser(String roleName, String username) throws RemoteException, UserNotFoundException,
      RoleNotFoundException {
    getAPI().addRoleToUser(roleName, username);
  }

  public void setUserRoles(String username, Set<String> roleNames) throws RemoteException, UserNotFoundException, RoleNotFoundException {
    getAPI().setUserRoles(username, roleNames);
  }
  
  public User addUser(String username, String password) throws RemoteException, UserAlreadyExistsException {
    return getAPI().addUser(username, password);
  }

  public User addUser(String username, String password, String firstName, String lastName, String email)
      throws RemoteException, UserAlreadyExistsException {
    return getAPI().addUser(username, password, firstName, lastName, email);
  }

  public Role getRole(String name) throws RemoteException, RoleNotFoundException {
    return getAPI().getRole(name);
  }

  public Set<Role> getRoles() throws RemoteException {
    return getAPI().getRoles();
  }

  public User getUser(String username) throws RemoteException, UserNotFoundException {
    return getAPI().getUser(username);
  }

  public Set<Role> getUserRoles(String username) throws RemoteException, UserNotFoundException {
    return getAPI().getUserRoles(username);
  }

  public Set<User> getUsers() throws RemoteException {
    return getAPI().getUsers();
  }

  public Set<User> getUsersInRole(String name) throws RemoteException, RoleNotFoundException {
    return getAPI().getUsersInRole(name);
  }

  public void removeRole(String name) throws RemoteException, RoleNotFoundException {
    getAPI().removeRole(name);
  }

  public void removeRoleFromUser(String roleName, String username) throws RemoteException, UserNotFoundException,
      RoleNotFoundException {
    getAPI().removeRoleFromUser(roleName, username);
  }

  public void removeUser(String username) throws RemoteException, UserNotFoundException {
    getAPI().removeUser(username);
  }

  public Role updateRole(String oldName, String name, String label, String description) throws RemoteException,
      RoleNotFoundException, RoleAlreadyExistsException {
    return getAPI().updateRole(oldName, name, label, description);
  }

  public User updateUser(String oldUsername, String username, String password, String firstName, String lastName,
      String email) throws RemoteException, UserNotFoundException, UserAlreadyExistsException {
    return getAPI().updateUser(oldUsername, username, password, firstName, lastName, email);
  }

}
