/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 * 
 * Modified by Matthieu Chaffotte - BonitaSoft S.A.
 **/
package org.ow2.bonita.facade.ejb;

import java.rmi.RemoteException;
import java.util.Collection;
import java.util.List;
import java.util.Set;

import javax.ejb.SessionBean;
import javax.ejb.SessionContext;

import org.ow2.bonita.facade.ManagementAPI;
import org.ow2.bonita.facade.def.element.BusinessArchive;
import org.ow2.bonita.facade.def.majorElement.ProcessDefinition;
import org.ow2.bonita.facade.exception.DeploymentException;
import org.ow2.bonita.facade.exception.ProcessNotFoundException;
import org.ow2.bonita.facade.exception.RuleAlreadyExistsException;
import org.ow2.bonita.facade.exception.RuleNotFoundException;
import org.ow2.bonita.facade.exception.UndeletableInstanceException;
import org.ow2.bonita.facade.exception.UndeletableProcessException;
import org.ow2.bonita.facade.exception.UserNotFoundException;
import org.ow2.bonita.facade.impl.StandardAPIAccessorImpl;
import org.ow2.bonita.facade.internal.InternalManagementAPI;
import org.ow2.bonita.facade.privilege.PrivilegePolicy;
import org.ow2.bonita.facade.privilege.Rule;
import org.ow2.bonita.facade.privilege.Rule.RuleType;
import org.ow2.bonita.facade.uuid.AbstractUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;

public class ManagementAPIBean implements InternalManagementAPI, SessionBean {

  private static final long serialVersionUID = 7816343722790053864L;

  protected SessionContext ctx = null;

  protected ManagementAPI managementAPI;

  protected ManagementAPI getAPI() {
    EJB2SecurityOwner.setUser(this.ctx.getCallerPrincipal().getName());
    if (this.managementAPI == null) {
      this.managementAPI = new StandardAPIAccessorImpl().getManagementAPI();
    }
    return this.managementAPI;
  }

  public void ejbCreate() {

  }

  public void ejbActivate() throws RemoteException {

  }

  public void ejbPassivate() throws RemoteException {

  }

  public void ejbRemove() throws RemoteException {

  }

  public void setSessionContext(final SessionContext arg0)
      throws RemoteException {
    this.ctx = arg0;
  }

  public ProcessDefinition deploy(final BusinessArchive businessArchive)
      throws DeploymentException {
    return getAPI().deploy(businessArchive);
  }

  public void deployJar(final String jarName, final byte[] jar)
      throws DeploymentException {
    getAPI().deployJar(jarName, jar);
  }

  public Set<String> getAvailableJars() throws RemoteException {
    return getAPI().getAvailableJars();
  }

  public void removeJar(final String jarName) throws DeploymentException {
    getAPI().removeJar(jarName);
  }

  public void deleteProcess(final ProcessDefinitionUUID processUUID)
      throws ProcessNotFoundException, UndeletableProcessException,
      UndeletableInstanceException {
    getAPI().deleteProcess(processUUID);
  }

  public void deleteAllProcesses() throws UndeletableInstanceException,
      RemoteException, UndeletableProcessException {
    getAPI().deleteAllProcesses();
  }

  public String getLoggedUser() throws RemoteException {
    return getAPI().getLoggedUser();
  }

  public void addMetaData(String key, String value) throws RemoteException {
    getAPI().addMetaData(key, value);
  }

  public void deleteMetaData(String key) throws RemoteException {
    getAPI().deleteMetaData(key);
  }

  public String getMetaData(String key) throws RemoteException {
    return getAPI().getMetaData(key);
  }

  public void archive(ProcessDefinitionUUID processUUID)
      throws DeploymentException {
    getAPI().archive(processUUID);
  }

  public void disable(ProcessDefinitionUUID processUUID)
      throws DeploymentException {
    getAPI().disable(processUUID);
  }

  public void enable(ProcessDefinitionUUID processUUID)
      throws DeploymentException {
    getAPI().enable(processUUID);
  }

  public void archive(Collection<ProcessDefinitionUUID> processUUIDs)
      throws DeploymentException, RemoteException {
    getAPI().archive(processUUIDs);
  }

  public void delete(Collection<ProcessDefinitionUUID> processUUIDs)
      throws DeploymentException, ProcessNotFoundException,
      UndeletableProcessException, UndeletableInstanceException,
      RemoteException {
    getAPI().delete(processUUIDs);
  }

  public void disable(Collection<ProcessDefinitionUUID> processUUIDs)
      throws DeploymentException, RemoteException {
    getAPI().disable(processUUIDs);
  }

  public void enable(Collection<ProcessDefinitionUUID> processUUIDs)
      throws DeploymentException, RemoteException {
    getAPI().enable(processUUIDs);
  }

  public boolean isUserAdmin(String username) throws UserNotFoundException,
      RemoteException {
    return getAPI().isUserAdmin(username);
  }

  public boolean checkUserCredentials(String username, String password)
      throws RemoteException {
    return getAPI().checkUserCredentials(username, password);
  }

  public <E extends AbstractUUID> void addExceptionsToRule(String ruleName, Set<E> exceptions)
      throws RemoteException, RuleNotFoundException {
    getAPI().addExceptionsToRule(ruleName, exceptions);
  }

  public void applyRulesToEntity(String ruleName,
      Set<String> rules) throws RemoteException, RuleNotFoundException {
    getAPI().applyRulesToEntity(ruleName, rules);

  }

  public void applySameRulesToOtherEntities(String entityID, Set<String> entities)
      throws RemoteException {
    getAPI().applySameRulesToOtherEntities(entityID, entities);

  }

  public <E extends AbstractUUID> Rule createRule(String name, String label, String description, Set<E> exceptions,
      RuleType type) throws RemoteException, RuleAlreadyExistsException {

    return getAPI().createRule(name, label, description, exceptions, type);
  }

  public void deleteRule(String name) throws RuleNotFoundException, RemoteException {
    getAPI().deleteRule(name);
  }


  public Rule getRule(String name) throws RuleNotFoundException, RemoteException {
    return getAPI().getRule(name);
  }

  public <E extends AbstractUUID> void removeExceptionsFromRule(String ruleName, Set<E> exceptions)
      throws RuleNotFoundException, RemoteException {
    getAPI().removeExceptionsFromRule(ruleName, exceptions);

  }

  public void removeRulesFromEntity(
      String ruleName, Set<String> rules) throws RemoteException, RuleNotFoundException {
    getAPI().removeRulesFromEntity(ruleName, rules);

  }

  public void updateRule(String name, Rule newRule)
      throws RuleNotFoundException, RemoteException {
    getAPI().updateRule(name, newRule);

  }
  
  public Set<Rule> getRules() throws RemoteException{
    return getAPI().getRules();
  }

  public Set<Rule> getRules(Set<String> ruleNames) throws RemoteException,
      RuleNotFoundException {
    return getAPI().getRules(ruleNames);
  }

  public Set<Rule> getAllApplicableRules(String entityID) throws RemoteException {
    return getAPI().getAllApplicableRules(entityID);
  }

  public Set<Rule> getApplicableRules(String entityID, RuleType ruleType) throws RemoteException {
    return getAPI().getApplicableRules(entityID, ruleType);
  }

  public PrivilegePolicy getDefaultPolicy() throws RemoteException{
   return getAPI().getDefaultPolicy();
  }

  public void setDefaultPolicy(PrivilegePolicy newPolicy, boolean negateRules) throws RemoteException{
    getAPI().setDefaultPolicy(newPolicy,negateRules);
  }

  public List<Rule> getRules(RuleType ruleType, int fromIndex, int pageSige) throws RemoteException {
    return getAPI().getRules(ruleType, fromIndex, pageSige);
  }

  public long getNumberOfRules(RuleType ruleType) throws RemoteException {
    return getAPI().getNumberOfRules(ruleType);
  }

}
