/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 * 
 * Modified by Charles Souillard, Matthieu Chaffotte, Nicolas Chabanoles - BonitaSoft S.A.
 **/
package org.ow2.bonita.facade.ejb;

import java.rmi.RemoteException;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.ejb.SessionBean;
import javax.ejb.SessionContext;

import org.ow2.bonita.facade.exception.ActivityNotFoundException;
import org.ow2.bonita.facade.exception.InstanceNotFoundException;
import org.ow2.bonita.facade.exception.ProcessNotFoundException;
import org.ow2.bonita.facade.exception.TaskNotFoundException;
import org.ow2.bonita.facade.exception.VariableNotFoundException;
import org.ow2.bonita.facade.impl.StandardInternalAPIAccessorImpl;
import org.ow2.bonita.facade.internal.InternalQueryRuntimeAPI;
import org.ow2.bonita.facade.runtime.ActivityInstance;
import org.ow2.bonita.facade.runtime.ActivityState;
import org.ow2.bonita.facade.runtime.AttachmentInstance;
import org.ow2.bonita.facade.runtime.Comment;
import org.ow2.bonita.facade.runtime.InstanceState;
import org.ow2.bonita.facade.runtime.ProcessInstance;
import org.ow2.bonita.facade.runtime.TaskInstance;
import org.ow2.bonita.facade.uuid.ActivityInstanceUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.light.LightActivityInstance;
import org.ow2.bonita.light.LightProcessInstance;
import org.ow2.bonita.light.LightTaskInstance;

/**
 * @author Pierre Vigneras, Charles Souillard, Guillaume Porcher
 */
public class QueryRuntimeAPIBean implements InternalQueryRuntimeAPI, SessionBean {

  private static final long serialVersionUID = -8407335961553034778L;

  protected SessionContext ctx = null;

  protected InternalQueryRuntimeAPI queryRuntimeAPI;

  protected InternalQueryRuntimeAPI getAPI() {
    EJB2SecurityOwner.setUser(this.ctx.getCallerPrincipal().getName());
    if (this.queryRuntimeAPI == null) {
      this.queryRuntimeAPI = new StandardInternalAPIAccessorImpl().getQueryRuntimeAPI();
    }
    return this.queryRuntimeAPI;
  }

  public void ejbCreate() {
  }

  public void ejbActivate() throws RemoteException {

  }

  public void ejbPassivate() throws RemoteException {

  }

  public void ejbRemove() throws RemoteException {

  }

  public void setSessionContext(final SessionContext arg0) throws RemoteException {
    this.ctx = arg0;
  }

  public Set<String> getTaskCandidates(ActivityInstanceUUID taskUUID, String queryList) throws RemoteException, TaskNotFoundException {
    return getAPI().getTaskCandidates(taskUUID, queryList);
  }
  
  public Map<ActivityInstanceUUID, Set<String>> getTaskCandidates(Set<ActivityInstanceUUID> taskUUIDs, String queryList) throws RemoteException, TaskNotFoundException {
    return getAPI().getTaskCandidates(taskUUIDs, queryList);
  }
  
  public List<LightProcessInstance> getLightParentProcessInstances(int fromIndex, int pageSize, String queryList)
  throws RemoteException {
    return getAPI().getLightParentProcessInstances(fromIndex, pageSize, queryList);
  }

  public int getNumberOfParentProcessInstances(String queryList)
  throws RemoteException {
    return getAPI().getNumberOfParentProcessInstances(queryList);
  }

  public int getNumberOfProcessInstances(String queryList)
  throws RemoteException {
    return getAPI().getNumberOfProcessInstances(queryList);
  }

  public List<LightActivityInstance> getLightActivityInstancesFromRoot(ProcessInstanceUUID rootInstanceUUID, String queryList)
  throws RemoteException {
    return getAPI().getLightActivityInstancesFromRoot(rootInstanceUUID, queryList);
  }

  public Map<ProcessInstanceUUID, List<LightActivityInstance>> getLightActivityInstancesFromRoot(Set<ProcessInstanceUUID> rootInstanceUUIDs, String queryList)
  throws RemoteException {
    return getAPI().getLightActivityInstancesFromRoot(rootInstanceUUIDs, queryList);
  }

  public List<LightTaskInstance> getLightTaskInstancesFromRoot(ProcessInstanceUUID rootInstanceUUID, String queryList)
  throws RemoteException {
    return getAPI().getLightTaskInstancesFromRoot(rootInstanceUUID, queryList);
  }

  public Map<ProcessInstanceUUID, List<LightTaskInstance>> getLightTaskInstancesFromRoot(Set<ProcessInstanceUUID> rootInstanceUUIDs, String queryList)
  throws RemoteException {
    return getAPI().getLightTaskInstancesFromRoot(rootInstanceUUIDs, queryList);
  }

  public ActivityInstanceUUID getOneTask(ProcessDefinitionUUID processUUID, ActivityState taskState, String queryList)
  throws RemoteException {
    return getAPI().getOneTask(processUUID, taskState, queryList);
  }

  public ActivityInstanceUUID getOneTask(ProcessInstanceUUID instanceUUID, ActivityState taskState, String queryList)
  throws RemoteException {
    return getAPI().getOneTask(instanceUUID, taskState, queryList);
  }

  public ActivityInstanceUUID getOneTask(ActivityState taskState, String queryList)
  throws RemoteException {
    return getAPI().getOneTask(taskState, queryList);
  }
  
  public Set<ActivityInstance> getActivityInstances(final ProcessInstanceUUID instanceUUID, final String queryList)
  throws InstanceNotFoundException, RemoteException {
    return getAPI().getActivityInstances(instanceUUID, queryList);
  }

  public Set<ActivityInstance> getActivityInstances(final ProcessInstanceUUID instanceUUID, final String activityId, final String queryList)
  throws InstanceNotFoundException, ActivityNotFoundException, RemoteException {
    return getAPI().getActivityInstances(instanceUUID, activityId, queryList);
  }

  public ActivityInstance getActivityInstance(final ActivityInstanceUUID activityUUID, final String queryList)
  throws ActivityNotFoundException, RemoteException {
    return getAPI().getActivityInstance(activityUUID, queryList);
  }

  public ProcessInstance getProcessInstance(final ProcessInstanceUUID instanceUUID, final String queryList)
  throws InstanceNotFoundException, RemoteException {
    return getAPI().getProcessInstance(instanceUUID, queryList);
  }

  public Set<ProcessInstance> getProcessInstances(final String queryList)
  throws RemoteException {
    return getAPI().getProcessInstances(queryList);
  }

  public Set<ProcessInstance> getProcessInstances(Collection<ProcessInstanceUUID> instanceUUIDs, String queryList)
  throws RemoteException {
    return getAPI().getProcessInstances(instanceUUIDs, queryList);
  }

  public Set<ProcessInstance> getProcessInstancesWithTaskState(Collection<ActivityState> activityStates, String queryList)
  throws RemoteException {
    return getAPI().getProcessInstancesWithTaskState(activityStates, queryList);
  }

  public boolean canExecuteTask(ActivityInstanceUUID taskUUID, String queryList)
  throws TaskNotFoundException, RemoteException {
    return getAPI().canExecuteTask(taskUUID, queryList);
  }
  
  public Set<ProcessInstance> getProcessInstances(final ProcessDefinitionUUID processUUID, final String queryList)
  throws ProcessNotFoundException, RemoteException {
    return getAPI().getProcessInstances(processUUID, queryList);
  }

  public TaskInstance getTask(final ActivityInstanceUUID taskUUID, final String queryList)
  throws TaskNotFoundException, RemoteException {
    return getAPI().getTask(taskUUID, queryList);
  }

  public Collection<TaskInstance> getTaskList(final ProcessInstanceUUID instanceUUID, final ActivityState taskState, final String queryList)
  throws InstanceNotFoundException, RemoteException {
    return getAPI().getTaskList(instanceUUID, taskState, queryList);
  }

  public Collection<TaskInstance> getTaskList(ProcessInstanceUUID instanceUUID, Collection<ActivityState> taskStates, String queryList)
  throws InstanceNotFoundException, RemoteException {
    return getAPI().getTaskList(instanceUUID, taskStates, queryList);
  }

  public Collection<TaskInstance> getTaskList(final ProcessInstanceUUID instanceUUID, final String userId, final ActivityState taskState, final String queryList)
  throws InstanceNotFoundException, RemoteException {
    return getAPI().getTaskList(instanceUUID, userId, taskState, queryList);
  }

  public Collection<TaskInstance> getTaskList(final ActivityState taskState, final String queryList)
  throws RemoteException {
    return getAPI().getTaskList(taskState, queryList);
  }

  public Collection<TaskInstance> getTaskList(final String userId, final ActivityState taskState, final String queryList)
  throws RemoteException {
    return getAPI().getTaskList(userId, taskState, queryList);
  }

  public Set<TaskInstance> getTasks(final ProcessInstanceUUID instanceUUID, final String queryList)
  throws InstanceNotFoundException, RemoteException {
    return getAPI().getTasks(instanceUUID, queryList);
  }

  public Object getActivityInstanceVariable(final ActivityInstanceUUID activityUUID, final String variableId, final String queryList)
  throws ActivityNotFoundException, VariableNotFoundException, RemoteException {
    return getAPI().getActivityInstanceVariable(activityUUID, variableId, queryList);
  }

  public ActivityState getActivityInstanceState(ActivityInstanceUUID activityUUID, String queryList)
  throws ActivityNotFoundException, RemoteException {
    return getAPI().getActivityInstanceState(activityUUID, queryList);
  }
  
  public Object getProcessInstanceVariable(final ProcessInstanceUUID instanceUUID, final String variableId, final String queryList)
  throws InstanceNotFoundException, VariableNotFoundException, RemoteException {
    return getAPI().getProcessInstanceVariable(instanceUUID, variableId, queryList);
  }

  public Map<String, Object> getActivityInstanceVariables(final ActivityInstanceUUID activityUUID, final String queryList)
  throws ActivityNotFoundException, RemoteException {
    return getAPI().getActivityInstanceVariables(activityUUID, queryList);
  }

  public Map<String, Object> getProcessInstanceVariables(final ProcessInstanceUUID instanceUUID, final String queryList)
  throws InstanceNotFoundException, RemoteException {
    return getAPI().getProcessInstanceVariables(instanceUUID, queryList);
  }

  public Object getVariable(final ActivityInstanceUUID activityUUID, final String variableId, final String queryList)
  throws ActivityNotFoundException, VariableNotFoundException, RemoteException {
    return getAPI().getVariable(activityUUID, variableId, queryList);
  }

  public Map<String, Object> getVariables(final ActivityInstanceUUID activityUUID, final String queryList)
  throws InstanceNotFoundException, ActivityNotFoundException, RemoteException {
    return getAPI().getVariables(activityUUID, queryList);
  }

  public List<Comment> getCommentFeed(ProcessInstanceUUID instanceUUID, String queryList)
  throws InstanceNotFoundException, RemoteException {
    return getAPI().getCommentFeed(instanceUUID, queryList);
  }

  public Set<ProcessInstance> getUserInstances(String queryList)
  throws RemoteException {
    return getAPI().getUserInstances(queryList);
  }

  public Set<String> getAttachmentNames(ProcessInstanceUUID instanceUUID, String queryList)
  throws RemoteException {
    return getAPI().getAttachmentNames(instanceUUID, queryList);
  }

  public List<AttachmentInstance> getAttachments(ProcessInstanceUUID instanceUUID, String attachmentName, String queryList) 
  throws RemoteException {
    return getAPI().getAttachments(instanceUUID, attachmentName, queryList);
  }

  public AttachmentInstance getLastAttachment(ProcessInstanceUUID instanceUUID, String attachmentName, String queryList)
  throws RemoteException {
    return getAPI().getLastAttachment(instanceUUID, attachmentName, queryList);
  }

  public Collection<AttachmentInstance> getLastAttachments(ProcessInstanceUUID instanceUUID, Set<String> attachmentNames, String queryList)
  throws RemoteException {
    return getAPI().getLastAttachments(instanceUUID, attachmentNames, queryList);
  }

  public AttachmentInstance getLastAttachment(ProcessInstanceUUID instanceUUID, String attachmentName, ActivityInstanceUUID activityUUID, String queryList)
  throws ActivityNotFoundException, RemoteException {
    return getAPI().getLastAttachment(instanceUUID, attachmentName, activityUUID, queryList);
  }

  public AttachmentInstance getLastAttachment(ProcessInstanceUUID instanceUUID, String attachmentName, Date date, String queryList)
  throws RemoteException {
    return getAPI().getLastAttachment(instanceUUID, attachmentName, date, queryList);
  }

  public Collection<AttachmentInstance> getLastAttachments(ProcessInstanceUUID instanceUUID, String regex, String queryList)
  throws RemoteException {
    return getAPI().getLastAttachments(instanceUUID, regex, queryList);
  }

  public byte[] getAttachmentValue(AttachmentInstance attachmentInstance, String queryList)
  throws RemoteException {
    return getAPI().getAttachmentValue(attachmentInstance, queryList);
  }

  public Set<LightProcessInstance> getLightProcessInstances(Collection<ProcessInstanceUUID> instanceUUIDs, String queryList)
  throws RemoteException {
    return getAPI().getLightProcessInstances(instanceUUIDs, queryList);
  }

  public List<LightProcessInstance> getLightProcessInstances(Collection<ProcessInstanceUUID> instanceUUIDs, int fromIndex, int pageSize, String queryList)
  throws RemoteException {
    return getAPI().getLightProcessInstances(instanceUUIDs, fromIndex, pageSize, queryList);
  }

  public Set<LightProcessInstance> getLightUserInstances(String queryList)
  throws RemoteException {
    return getAPI().getLightUserInstances(queryList);
  }

  public LightProcessInstance getLightProcessInstance(ProcessInstanceUUID instanceUUID, String queryList)
  throws InstanceNotFoundException, RemoteException {
    return getAPI().getLightProcessInstance(instanceUUID, queryList);
  }

  public Set<LightProcessInstance> getLightProcessInstances(ProcessDefinitionUUID processUUID, String queryList)
  throws RemoteException {
    return getAPI().getLightProcessInstances(processUUID, queryList);
  }

  public Set<LightProcessInstance> getLightWeightProcessInstances(Set<ProcessDefinitionUUID> processUUIDs, String queryList)
  throws RemoteException {
    return getAPI().getLightWeightProcessInstances(processUUIDs, queryList);
  }

  public Set<LightProcessInstance> getLightProcessInstances(String queryList)
  throws RemoteException {
    return getAPI().getLightProcessInstances(queryList);
  }

  public List<LightProcessInstance> getLightProcessInstances(int fromIndex, int pageSize, String queryList)
  throws RemoteException {
    return getAPI().getLightProcessInstances(fromIndex, pageSize, queryList);
  }

  public Set<LightActivityInstance> getLightActivityInstances(ProcessInstanceUUID instanceUUID, String queryList)
  throws InstanceNotFoundException, RemoteException {
    return getAPI().getLightActivityInstances(instanceUUID, queryList);
  }

  public Set<LightTaskInstance> getLightTasks(ProcessInstanceUUID instanceUUID,String queryList)
  throws InstanceNotFoundException, RemoteException {
    return getAPI().getLightTasks(instanceUUID, queryList);
  }

  public List<Comment> getActivityInstanceCommentFeed(ActivityInstanceUUID activityUUID, String queryList)
  throws InstanceNotFoundException, RemoteException {
    return getAPI().getActivityInstanceCommentFeed(activityUUID, queryList);
  }

  public List<Comment> getProcessInstanceCommentFeed(ProcessInstanceUUID instanceUUID, String queryList)
  throws InstanceNotFoundException, RemoteException {
    return getAPI().getProcessInstanceCommentFeed(instanceUUID, queryList);
  }

  public int getNumberOfActivityInstanceComments(ActivityInstanceUUID activityUUID, String queryList)
  throws InstanceNotFoundException, RemoteException {
    return getAPI().getNumberOfActivityInstanceComments(activityUUID, queryList);
  }
  
  public Map<ActivityInstanceUUID, Integer> getNumberOfActivityInstanceComments(Set<ActivityInstanceUUID> activityUUIDs, String queryList)
  throws InstanceNotFoundException, RemoteException {
	  return getAPI().getNumberOfActivityInstanceComments(activityUUIDs, queryList);
  }

  public int getNumberOfComments(ProcessInstanceUUID instanceUUID, String queryList)
  throws InstanceNotFoundException, RemoteException {
    return getAPI().getNumberOfComments(instanceUUID, queryList);
  }

  public int getNumberOfProcessInstanceComments(ProcessInstanceUUID instanceUUID, String queryList)
  throws InstanceNotFoundException, RemoteException {
    return getAPI().getNumberOfProcessInstanceComments(instanceUUID, queryList);
  }

  public LightTaskInstance getLightTaskInstance(ActivityInstanceUUID taskUUID, String queryList)
  throws TaskNotFoundException, RemoteException {
    return getAPI().getLightTaskInstance(taskUUID, queryList);
  }

  public Set<LightActivityInstance> getLightActivityInstances(ProcessInstanceUUID instanceUUID, String activityName, String queryList)
  throws InstanceNotFoundException, ActivityNotFoundException, RemoteException {
    return getAPI().getLightActivityInstances(instanceUUID, activityName, queryList);
  }

  public Set<ProcessInstance> getProcessInstancesWithInstanceStates(Collection<InstanceState> instanceStates, String queryList) throws RemoteException {
    return getAPI().getProcessInstancesWithInstanceStates(instanceStates, queryList);
  }

  public LightActivityInstance getLightActivityInstance(ActivityInstanceUUID activityInstanceUUID, String queryList)
      throws ActivityNotFoundException, RemoteException {
    return getAPI().getLightActivityInstance(activityInstanceUUID, queryList);
  }
  
  public Collection<LightTaskInstance> getLightTaskList(ActivityState taskState, String queryList) throws RemoteException {
    return getAPI().getLightTaskList(taskState, queryList);
  }
  public Collection<LightTaskInstance> getLightTaskList(ProcessInstanceUUID instanceUUID, ActivityState taskState, String queryList) throws InstanceNotFoundException, RemoteException {
    return getAPI().getLightTaskList(instanceUUID, taskState, queryList);
  }
  public Collection<LightTaskInstance> getLightTaskList(ProcessInstanceUUID instanceUUID, Collection<ActivityState> taskStates, String queryList) throws InstanceNotFoundException, RemoteException {
    return getAPI().getLightTaskList(instanceUUID, taskStates, queryList);
  }
  public Collection<LightTaskInstance> getLightTaskList(ProcessInstanceUUID instanceUUID, String userId, ActivityState taskState, String queryList) throws InstanceNotFoundException, RemoteException {
    return getAPI().getLightTaskList(instanceUUID, userId, taskState, queryList);
  }
  public Collection<LightTaskInstance> getLightTaskList(String userId, ActivityState taskState, String queryList) throws RemoteException {
    return getAPI().getLightTaskList(userId, taskState, queryList);
  }

  public Map<ProcessInstanceUUID, List<LightActivityInstance>> getLightActivityInstancesFromRoot(Set<ProcessInstanceUUID> rootInstanceUUIDs, ActivityState state, String queryList) throws RemoteException {
    return getAPI().getLightActivityInstancesFromRoot(rootInstanceUUIDs, state, queryList);
  }
  public Map<ProcessInstanceUUID, LightActivityInstance> getLightLastUpdatedActivityInstanceFromRoot(Set<ProcessInstanceUUID> rootInstanceUUIDs, boolean considerSystemTaks, String queryList) throws RemoteException {
    return getAPI().getLightLastUpdatedActivityInstanceFromRoot(rootInstanceUUIDs, considerSystemTaks, queryList);
  }
}
