/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 * 
 * Modified by Matthieu Chaffotte - BonitaSoft S.A.
 **/
package org.ow2.bonita.facade.ejb;

import java.rmi.RemoteException;
import java.util.Collection;
import java.util.Map;
import java.util.Set;

import javax.ejb.SessionBean;
import javax.ejb.SessionContext;

import org.ow2.bonita.facade.RuntimeAPI;
import org.ow2.bonita.facade.exception.ActivityNotFoundException;
import org.ow2.bonita.facade.exception.IllegalTaskStateException;
import org.ow2.bonita.facade.exception.InstanceNotFoundException;
import org.ow2.bonita.facade.exception.ProcessNotFoundException;
import org.ow2.bonita.facade.exception.TaskNotFoundException;
import org.ow2.bonita.facade.exception.UncancellableInstanceException;
import org.ow2.bonita.facade.exception.UndeletableInstanceException;
import org.ow2.bonita.facade.exception.VariableNotFoundException;
import org.ow2.bonita.facade.impl.StandardAPIAccessorImpl;
import org.ow2.bonita.facade.internal.InternalRuntimeAPI;
import org.ow2.bonita.facade.runtime.AttachmentInstance;
import org.ow2.bonita.facade.runtime.InitialAttachment;
import org.ow2.bonita.facade.uuid.ActivityInstanceUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.util.GroovyException;

public class RuntimeAPIBean implements InternalRuntimeAPI, SessionBean {

  private static final long serialVersionUID = 4856106680913239049L;

  protected SessionContext ctx = null;

  protected RuntimeAPI runtimeAPI;

  protected RuntimeAPI getAPI() {
    EJB2SecurityOwner.setUser(this.ctx.getCallerPrincipal().getName());
    if (this.runtimeAPI == null) {
      this.runtimeAPI = new StandardAPIAccessorImpl().getRuntimeAPI();
    }
    return this.runtimeAPI;
  }

  public void ejbCreate() {

  }

  public void ejbActivate() throws RemoteException {

  }

  public void ejbPassivate() throws RemoteException {

  }

  public void ejbRemove() throws RemoteException {

  }

  public void setSessionContext(final SessionContext arg0) throws RemoteException {
    this.ctx = arg0;
  }

  public void enableEventsInFailure(ProcessInstanceUUID instanceUUID, String activityName) {
    getAPI().enableEventsInFailure(instanceUUID, activityName);
  }
  
  public void enableEventsInFailure(ActivityInstanceUUID activityUUID) {
    getAPI().enableEventsInFailure(activityUUID);
  }
  
  public void startTask(final ActivityInstanceUUID taskUUID,
      final boolean assignTask) throws TaskNotFoundException, IllegalTaskStateException {
    getAPI().startTask(taskUUID, assignTask);
  }

  public void finishTask(final ActivityInstanceUUID taskUUID,
      final boolean taskAssign) throws TaskNotFoundException, IllegalTaskStateException {
    getAPI().finishTask(taskUUID, taskAssign);
  }

  public void resumeTask(final ActivityInstanceUUID taskUUID, final boolean taskAssign) throws TaskNotFoundException,
  IllegalTaskStateException {
    getAPI().resumeTask(taskUUID, taskAssign);
  }

  public void startActivity(ActivityInstanceUUID activityUUID) throws ActivityNotFoundException, RemoteException {
    getAPI().startActivity(activityUUID);
  }
  
  public void suspendTask(final ActivityInstanceUUID taskUUID, final boolean assignTask) throws TaskNotFoundException,
  IllegalTaskStateException {
    getAPI().suspendTask(taskUUID, assignTask);
  }

  public void assignTask(final ActivityInstanceUUID taskUUID) throws TaskNotFoundException {
    getAPI().assignTask(taskUUID);
  }

  public void assignTask(final ActivityInstanceUUID taskUUID, final String actorId) throws TaskNotFoundException {
    getAPI().assignTask(taskUUID, actorId);
  }

  public void assignTask(final ActivityInstanceUUID taskUUID, final Set<String> candidates) throws TaskNotFoundException {
    getAPI().assignTask(taskUUID, candidates);
  }

  public void unassignTask(final ActivityInstanceUUID taskUUID) throws TaskNotFoundException {
    getAPI().unassignTask(taskUUID);
  }

  public void deleteAllProcessInstances(final Collection<ProcessDefinitionUUID> processUUIDs) throws ProcessNotFoundException,
    UndeletableInstanceException {
    getAPI().deleteAllProcessInstances(processUUIDs);
  }
  
  public void deleteAllProcessInstances(final ProcessDefinitionUUID processUUID) throws ProcessNotFoundException,
    UndeletableInstanceException {
    getAPI().deleteAllProcessInstances(processUUID);
  }

  public void executeTask(ActivityInstanceUUID taskUUID, boolean assignTask)
      throws TaskNotFoundException, IllegalTaskStateException, RemoteException {
    getAPI().executeTask(taskUUID, assignTask);
  }
  public void deleteProcessInstance(final ProcessInstanceUUID instanceUUID)
    throws InstanceNotFoundException, UndeletableInstanceException {
    getAPI().deleteProcessInstance(instanceUUID);
  }
  
  public void deleteProcessInstances(Collection<ProcessInstanceUUID> instanceUUIDs)
      throws InstanceNotFoundException, UndeletableInstanceException {
    getAPI().deleteProcessInstances(instanceUUIDs);
  }

  public void cancelProcessInstance(final ProcessInstanceUUID instanceUUID) throws InstanceNotFoundException,
      UncancellableInstanceException {
    getAPI().cancelProcessInstance(instanceUUID);
  }
  
  public void cancelProcessInstances(Collection<ProcessInstanceUUID> instanceUUIDs)
  throws InstanceNotFoundException, UncancellableInstanceException {
    getAPI().cancelProcessInstances(instanceUUIDs);
  }

  public ProcessInstanceUUID instantiateProcess(
      final ProcessDefinitionUUID processUUID) throws ProcessNotFoundException {
    return getAPI().instantiateProcess(processUUID);
  }

  public ProcessInstanceUUID instantiateProcess(final ProcessDefinitionUUID processUUID, final Map<String, Object> variables)
  throws ProcessNotFoundException, VariableNotFoundException {
    return getAPI().instantiateProcess(processUUID, variables);
  }
  
  public ProcessInstanceUUID instantiateProcess(final ProcessDefinitionUUID processUUID, final Map<String, Object> variables,
      Collection<InitialAttachment> attachments) throws ProcessNotFoundException, VariableNotFoundException {
    return getAPI().instantiateProcess(processUUID, variables, attachments);
  }

  public void setProcessInstanceVariable(final ProcessInstanceUUID instanceUUID, final String variableId,
      final Object variableValue) throws InstanceNotFoundException, VariableNotFoundException {
    getAPI().setProcessInstanceVariable(instanceUUID, variableId, variableValue);
  }

  public void setActivityInstanceVariable(final ActivityInstanceUUID activityUUID,
      final String variableId, final Object variableValue) throws ActivityNotFoundException, VariableNotFoundException {
    getAPI().setActivityInstanceVariable(activityUUID, variableId, variableValue);
  }

  public void setVariable(final ActivityInstanceUUID activityUUID, final String variableId, final Object variableValue)
    throws ActivityNotFoundException, VariableNotFoundException {
    getAPI().setVariable(activityUUID, variableId, variableValue);
  }

  @Deprecated
  public void addComment(final ProcessInstanceUUID instanceUUID, 
     final  ActivityInstanceUUID activityUUID, final String message, final String userId)
  throws InstanceNotFoundException, ActivityNotFoundException {
    getAPI().addComment(instanceUUID, activityUUID, message, userId);
  }

  public void addComment(ProcessInstanceUUID instanceUUID, String message,
      String userId) throws InstanceNotFoundException, RemoteException {
    getAPI().addComment(instanceUUID, message, userId);
  }

  public void addComment(ActivityInstanceUUID activityUUID, String message,
      String userId) throws ActivityNotFoundException,
      InstanceNotFoundException, RemoteException {
    getAPI().addComment(activityUUID, message, userId);
  }
  
  public void addProcessMetaData(ProcessDefinitionUUID uuid, String key,
      String value) throws ProcessNotFoundException {
    getAPI().addProcessMetaData(uuid, key, value);
  }

  public void deleteProcessMetaData(ProcessDefinitionUUID uuid, String key)
      throws ProcessNotFoundException {
    getAPI().deleteProcessMetaData(uuid, key);
  }

  public Object evaluateGroovyExpression(String expression,
      ProcessInstanceUUID instanceUUID, boolean propagate) throws InstanceNotFoundException, GroovyException {
    return getAPI().evaluateGroovyExpression(expression, instanceUUID, propagate);
  }

  public Object evaluateGroovyExpression(String expression, ProcessInstanceUUID processInstanceUUID, Map<String, Object> context,
      boolean propagate) throws InstanceNotFoundException, GroovyException, RemoteException {
    return getAPI().evaluateGroovyExpression(expression, processInstanceUUID, context, propagate);
  }

  public Object evaluateGroovyExpression(String expression,
      ActivityInstanceUUID activityUUID, boolean propagate) throws InstanceNotFoundException,
      ActivityNotFoundException, GroovyException {
    return getAPI().evaluateGroovyExpression(expression, activityUUID, propagate);
  }

  public Object evaluateGroovyExpression(String expression,
      ActivityInstanceUUID activityInstanceUUID, Map<String, Object> context,
      boolean propagate) throws InstanceNotFoundException,
      ActivityNotFoundException, GroovyException, RemoteException {
    return getAPI().evaluateGroovyExpression(expression, activityInstanceUUID, context, propagate);
  }

  public Object evaluateGroovyExpression(String expression,
      ProcessDefinitionUUID processDefinitionUUID)
      throws ProcessNotFoundException, GroovyException, RemoteException {
    return getAPI().evaluateGroovyExpression(expression, processDefinitionUUID);
  }
  
  public Object evaluateGroovyExpression(String expression,
      ProcessDefinitionUUID processDefinitionUUID, Map<String, Object> context) throws ProcessNotFoundException, GroovyException,
      RemoteException {
    return getAPI().evaluateGroovyExpression(expression, processDefinitionUUID, context);
  }

  public void addAttachment(ProcessInstanceUUID instanceUUID, String name, String fileName, byte[] value) throws RemoteException {
    getAPI().addAttachment(instanceUUID, name, fileName, value);
  }

  public void addAttachment(ProcessInstanceUUID instanceUUID, String name, String label, String description, String fileName,
      Map<String, String> metadata, byte[] value) throws RemoteException {
    getAPI().addAttachment(instanceUUID, name, label, description, fileName, metadata, value); 
  }

  public void addAttachments(Map<AttachmentInstance, byte[]> attachments) throws RemoteException {
   getAPI().addAttachments(attachments); 
  }

  public void deleteEvents(String eventName, String toProcessName, String toActivityName, ActivityInstanceUUID actiivtyUUID)
  throws RemoteException {
    getAPI().deleteEvents(eventName, toProcessName, toActivityName, actiivtyUUID);
  }

  public Map<String, Object> executeConnector(String connectorClassName,
      Map<String, Object[]> parameters, ProcessDefinitionUUID definitionUUID)
      throws RemoteException, Exception {
    return getAPI().executeConnector(connectorClassName, parameters, definitionUUID);
  }

  public Map<String, Object> executeConnector(String connectorClassName,
      Map<String, Object[]> parameters) throws RemoteException, Exception {
    return getAPI().executeConnector(connectorClassName, parameters);
  }

  public Set<String> executeFilter(String connectorClassName,
      Map<String, Object[]> parameters, Set<String> members,
      ProcessDefinitionUUID definitionUUID) throws RemoteException, Exception {
    return getAPI().executeFilter(connectorClassName, parameters, members, definitionUUID);
  }

  public Set<String> executeFilter(String connectorClassName,
      Map<String, Object[]> parameters, Set<String> members)
      throws RemoteException, Exception {
    return getAPI().executeFilter(connectorClassName, parameters, members);
  }

  public Set<String> executeRoleResolver(String connectorClassName,
      Map<String, Object[]> parameters, ProcessDefinitionUUID definitionUUID)
      throws RemoteException, Exception {
    return getAPI().executeRoleResolver(connectorClassName, parameters, definitionUUID);
  }

  public Set<String> executeRoleResolver(String connectorClassName,
      Map<String, Object[]> parameters) throws RemoteException, Exception {
    return getAPI().executeRoleResolver(connectorClassName, parameters);
  }

  public Map<String, Object> executeConnector(String connectorClassName,
      Map<String, Object[]> parameters, ClassLoader classLoader) throws RemoteException, Exception {
    return getAPI().executeConnector(connectorClassName, parameters, classLoader);
  }

  public Set<String> executeFilter(String connectorClassName,
      Map<String, Object[]> parameters, Set<String> members, ClassLoader classLoader) throws RemoteException, Exception {
    return getAPI().executeFilter(connectorClassName, parameters, members, classLoader);
  }

  public Set<String> executeRoleResolver(String connectorClassName,
      Map<String, Object[]> parameters, ClassLoader classLoader)
      throws RemoteException, Exception {
    return getAPI().executeRoleResolver(connectorClassName, parameters, classLoader);
  }

  public void setActivityInstancePriority(ActivityInstanceUUID activityInstanceUUID, int priority)
      throws ActivityNotFoundException {
    getAPI().setActivityInstancePriority(activityInstanceUUID, priority);
  }

}
