/**
 * Copyright (C) 2009  BonitaSoft S.A.
 * BonitaSoft, 31 rue Gustave Eiffel - 38000 Grenoble
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade.ejb;

import java.rmi.RemoteException;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.ejb.SessionBean;
import javax.ejb.SessionContext;

import org.ow2.bonita.facade.WebAPI;
import org.ow2.bonita.facade.impl.StandardAPIAccessorImpl;
import org.ow2.bonita.facade.internal.InternalWebAPI;
import org.ow2.bonita.facade.runtime.Label;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.light.LightProcessInstance;

public class WebAPIBean implements InternalWebAPI, SessionBean {

  private static final long serialVersionUID = 4856106680913239049L;

  protected SessionContext ctx = null;

  protected WebAPI webAPI;

  protected WebAPI getAPI() {
    EJB2SecurityOwner.setUser(this.ctx.getCallerPrincipal().getName());
    if (this.webAPI == null) {
      this.webAPI = new StandardAPIAccessorImpl().getWebAPI();
    }
    return this.webAPI;
  }

  public void ejbCreate() { }
  public void ejbActivate() throws RemoteException { }
  public void ejbPassivate() throws RemoteException { }
  public void ejbRemove() throws RemoteException { }

  public void setSessionContext(final SessionContext arg0) throws RemoteException {
    this.ctx = arg0;
  }

  public void deletePhantomCases() throws RemoteException {
    getAPI().deletePhantomCases();
  }

  public void addCasesToLabel(String ownerName, String labelName, Set<ProcessInstanceUUID> caseList) throws RemoteException {
    getAPI().addCasesToLabel(ownerName, labelName, caseList);
  }

  public void addLabel(String labelName, String ownerName,
      String editableCSSStyleName, String readonlyCSSStyleName,
      String previewCSSStyleName, boolean isVisible, boolean hasToBeDisplayed,
      String iconCSSStyle, Set<ProcessInstanceUUID> caseList, int displayOrder,
      boolean isSystemLabel) throws RemoteException {
    getAPI().addLabel(labelName, ownerName, editableCSSStyleName, readonlyCSSStyleName, previewCSSStyleName, isVisible, hasToBeDisplayed, iconCSSStyle, caseList, displayOrder, isSystemLabel);
  }

  public Label getLabel(String ownerName, String labelName) throws RemoteException {
    return getAPI().getLabel(ownerName, labelName);
  }

  public Set<Label> getLabels(String ownerName) throws RemoteException {
    return getAPI().getLabels(ownerName);
  }

  public List<Label> getSystemLabels(String ownerName) throws RemoteException {
    return getAPI().getSystemLabels(ownerName);
  }

  public List<Label> getUserCustomLabels(String ownerName) throws RemoteException {
    return getAPI().getUserCustomLabels(ownerName);
  }

  public List<LightProcessInstance> getLightProcessInstances(String ownerName, Set<String> labelNames, int startingIndex, int maxElementCount) throws RemoteException {
    return getAPI().getLightProcessInstances(ownerName, labelNames, startingIndex, maxElementCount);
  }

  public Set<ProcessInstanceUUID> getCases(String ownerName, Set<String> labels) throws RemoteException {
    return getAPI().getCases(ownerName, labels);
  }

  public void removeCasesFromLabel(String ownerName, String labelName, Set<ProcessInstanceUUID> caseList) throws RemoteException {
    getAPI().removeCasesFromLabel(ownerName, labelName, caseList);
  }

  public void removeLabel(String ownerName, String labelName) throws RemoteException {
    getAPI().removeLabel(ownerName, labelName);
  }

  public void updateLabelCSS(String ownerName, String labelName,
      String aEditableCSSStyle, String aPreviewCSSStyle,
      String aReadOnlyCSSStyle) throws RemoteException {
    getAPI().updateLabelCSS(ownerName, labelName, aEditableCSSStyle, aPreviewCSSStyle, aReadOnlyCSSStyle);
  }

  public void updateLabelName(String ownerName, String labelName, String newName) throws RemoteException {
    getAPI().updateLabelName(ownerName, labelName, newName);
  }

  public Set<Label> getCaseLabels(String ownerName, ProcessInstanceUUID case_) throws RemoteException {
    return getAPI().getCaseLabels(ownerName, case_);
  }

  public Map<ProcessInstanceUUID, Set<Label>> getCasesLabels(String ownerName, Set<ProcessInstanceUUID> cases) throws RemoteException {
    return getAPI().getCasesLabels(ownerName, cases);
  }

  public void removeAllCasesFromLabels(Set<ProcessInstanceUUID> caseList) throws RemoteException {
    getAPI().removeAllCasesFromLabels(caseList);
  }

  public boolean setLabelsOnNewOrUpdatedCases(String ownerName, Set<String> labelsToApplyOnNewOrUpdated, Set<String> labelsToRemoveOnNewOrUpdated, String labelToApplyOnMyCases)
  throws RemoteException {
    return getAPI().setLabelsOnNewOrUpdatedCases(ownerName, labelsToApplyOnNewOrUpdated, labelsToRemoveOnNewOrUpdated, labelToApplyOnMyCases);
  }

  public Set<Label> getLabels(String ownerName, Set<String> labelsName) throws RemoteException {
    return getAPI().getLabels(ownerName, labelsName);
  }

  public void deleteAllCases() throws RemoteException {
    getAPI().deleteAllCases(); 
  }

  public void removeLabels(String ownerName, Collection<String> labelNames) throws RemoteException {
    getAPI().removeLabels(ownerName, labelNames);
  }

  public void updateLabelVisibility(String ownerName, String labelName, boolean isVisible)
  throws RemoteException {
    getAPI().updateLabelVisibility(ownerName, labelName, isVisible);
  }

  public void updateLabelVisibility(String ownerName, Map<String, Boolean> labelvisibilities)
  throws RemoteException {
    getAPI().updateLabelVisibility(ownerName, labelvisibilities);
  }
  public Map<String, Integer> getCasesNumber(String ownerName, Collection<String> labelNames, int limit) throws RemoteException {
    return getAPI().getCasesNumber(ownerName, labelNames, limit);
  }
  public Map<String, Integer> getCasesNumber(String ownerName, Collection<String> labelNames) throws RemoteException {
    return getAPI().getCasesNumber(ownerName, labelNames);
  }

  public Map<String, Integer> getCasesNumber(String ownerName, String labelName, Collection<String> labelNames, int limit) throws RemoteException {
    return getAPI().getCasesNumber(ownerName, labelName, labelNames, limit);
  }

  public String generateTemporaryToken(String identityKey) throws RemoteException {
    return getAPI().generateTemporaryToken(identityKey);
  }

  public String getIdentityKeyFromTemporaryToken(String token) throws RemoteException {
    return getAPI().getIdentityKeyFromTemporaryToken(token);
  }

}
