/**
 * Copyright (C) 2009  BonitaSoft S.A.
 * BonitaSoft, 31 rue Gustave Eiffel - 38000 Grenoble
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade.impl;

import java.rmi.RemoteException;
import java.util.Date;
import java.util.List;

import org.ow2.bonita.facade.internal.InternalBAMAPI;
import org.ow2.bonita.util.DateUtil;
import org.ow2.bonita.util.EnvTool;

/**
 * 
 * @author Matthieu Chaffotte
 *
 */
public class BAMAPIImpl implements InternalBAMAPI {

  protected BAMAPIImpl() {}

  public List<Integer> getNumberOfExecutingCasesPerDay(Date since, String queryList) throws RemoteException {
    Date now = new Date();
    return EnvTool.getAllQueriers(queryList).getNumberOfExecutingCasesPerDay(since, now);
  }

  public List<Integer> getNumberOfFinishedCasesPerDay(Date since, String queryList) throws RemoteException {
    Date now = new Date();
    return EnvTool.getAllQueriers(queryList).getNumberOfFinishedCasesPerDay(since, now);
  }

  public int getNumberOfOpenSteps(String queryList) throws RemoteException {
    return EnvTool.getJournalQueriers(queryList).getNumberOfOpenSteps();
  }

  public List<Integer> getNumberOfOpenStepsPerDay(Date since, String queryList) throws RemoteException {
    Date now = new Date();
    return EnvTool.getAllQueriers(queryList).getNumberOfOpenStepsPerDay(since, now);
  }

  public int getNumberOfOverdueSteps(String queryList) throws RemoteException {
    Date now = new Date();
    return EnvTool.getJournalQueriers(queryList).getNumberOfOverdueSteps(now);
  }

  public int getNumberOfStepsAtRisk(int remainingDays, String queryList) throws RemoteException {
    if (remainingDays < 0) {
      throw new IllegalArgumentException("The number of remaining days is negative");
    }
    Date currentDate = new Date();
    Date beginningOfTheDay = DateUtil.getBeginningOfTheDay(currentDate);
    Date atRisk = DateUtil.backTo(beginningOfTheDay, -(remainingDays+1));
    return EnvTool.getJournalQueriers(queryList).getNumberOfStepsAtRisk(currentDate, atRisk);
  }

  public int getNumberOfUserOpenSteps(String queryList) throws RemoteException {
    String userId = EnvTool.getUserId();
    return EnvTool.getJournalQueriers(queryList).getNumberOfUserOpenSteps(userId);
  }

  public int getNumberOfUserOverdueSteps(String queryList) throws RemoteException {
    Date now = new Date();
    String userId = EnvTool.getUserId();
    return EnvTool.getJournalQueriers(queryList).getNumberOfUserOverdueSteps(userId, now);
  }

  public int getNumberOfUserStepsAtRisk(int remainingDays, String queryList) throws RemoteException {
    if (remainingDays < 0) {
      throw new IllegalArgumentException("The number of remaining days is negative");
    }
    String userId = EnvTool.getUserId();
    Date currentDate = new Date();
    Date beginningOfTheDay = DateUtil.getBeginningOfTheDay(currentDate);
    Date atRisk = DateUtil.backTo(beginningOfTheDay, -(remainingDays+1));
    return EnvTool.getJournalQueriers(queryList).getNumberOfUserStepsAtRisk(userId, currentDate, atRisk);
  }

  public int getNumberOfFinishedSteps(int priority, Date since, String queryList) throws RemoteException {
    return EnvTool.getAllQueriers(queryList).getNumberOfFinishedSteps(priority, since);
  }

  public int getNumberOfOpenSteps(int priority, String queryList) throws RemoteException {
    return EnvTool.getJournalQueriers(queryList).getNumberOfOpenSteps(priority);
  }

  public int getNumberOfUserFinishedSteps(int priority, Date since, String queryList) throws RemoteException {
    String userId = EnvTool.getUserId();
    return EnvTool.getAllQueriers(queryList).getNumberOfUserFinishedSteps(userId, priority, since);
  }

  public int getNumberOfUserOpenSteps(int priority, String queryList) throws RemoteException {
    String userId = EnvTool.getUserId();
    return EnvTool.getJournalQueriers(queryList).getNumberOfUserOpenSteps(userId, priority);
  }
}
