/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 * 
 * Modified by Charles Souillard, Matthieu Chaffotte - BonitaSoft S.A.
 **/
package org.ow2.bonita.facade.impl;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.ow2.bonita.env.Environment;
import org.ow2.bonita.facade.exception.ActivityNotFoundException;
import org.ow2.bonita.facade.exception.InstanceNotFoundException;
import org.ow2.bonita.facade.exception.TaskNotFoundException;
import org.ow2.bonita.facade.exception.VariableNotFoundException;
import org.ow2.bonita.facade.internal.InternalQueryRuntimeAPI;
import org.ow2.bonita.facade.privilege.Rule.RuleType;
import org.ow2.bonita.facade.runtime.ActivityInstance;
import org.ow2.bonita.facade.runtime.ActivityState;
import org.ow2.bonita.facade.runtime.AttachmentInstance;
import org.ow2.bonita.facade.runtime.Comment;
import org.ow2.bonita.facade.runtime.InstanceState;
import org.ow2.bonita.facade.runtime.ProcessInstance;
import org.ow2.bonita.facade.runtime.TaskInstance;
import org.ow2.bonita.facade.runtime.VariableUpdate;
import org.ow2.bonita.facade.runtime.impl.ActivityInstanceImpl;
import org.ow2.bonita.facade.runtime.impl.AttachmentInstanceImpl;
import org.ow2.bonita.facade.runtime.impl.InternalActivityInstance;
import org.ow2.bonita.facade.runtime.impl.InternalProcessInstance;
import org.ow2.bonita.facade.runtime.impl.ProcessInstanceImpl;
import org.ow2.bonita.facade.uuid.ActivityInstanceUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.light.LightActivityInstance;
import org.ow2.bonita.light.LightProcessInstance;
import org.ow2.bonita.light.LightTaskInstance;
import org.ow2.bonita.light.impl.LightActivityInstanceImpl;
import org.ow2.bonita.light.impl.LightProcessInstanceImpl;
import org.ow2.bonita.services.LargeDataRepository;
import org.ow2.bonita.services.Querier;
import org.ow2.bonita.util.CopyTool;
import org.ow2.bonita.util.EnvTool;
import org.ow2.bonita.util.ExceptionManager;
import org.ow2.bonita.util.Misc;

/**
 * @author Pierre Vigneras
 */
public class QueryRuntimeAPIImpl implements InternalQueryRuntimeAPI {

	/*
  public QueryRuntimeAPIImpl(String CONF_NAME) {
    this.confName = CONF_NAME;
  }
	 */
  protected QueryRuntimeAPIImpl() {
	}

	public int getNumberOfParentProcessInstances(String queryList) throws RemoteException {
	  Environment env = Environment.getCurrent();
    boolean access = env.isRestrictedApplicationAcces();
    int count = 0;
    if (access) {
      String applicationName = env.getApplicationAccessName();
      Set<ProcessDefinitionUUID> visibleProcessUUIDs = FacadeUtil.getAllowedProcessUUIDsFor(applicationName, RuleType.PROCESS_READ);
      if (visibleProcessUUIDs != null) {
        count = EnvTool.getAllQueriers(queryList).getNumberOfParentProcessInstances(visibleProcessUUIDs);
      }
    } else {
      count = EnvTool.getAllQueriers(queryList).getNumberOfParentProcessInstances();
    }
    return count;
	}

	public int getNumberOfProcessInstances(String queryList) throws RemoteException {
	  Environment env = Environment.getCurrent();
    boolean access = env.isRestrictedApplicationAcces();
    int count = 0;
    if (access) {
      String applicationName = env.getApplicationAccessName();
      Set<ProcessDefinitionUUID> visibleProcessUUIDs = FacadeUtil.getAllowedProcessUUIDsFor(applicationName, RuleType.PROCESS_READ);
      if (visibleProcessUUIDs != null && !visibleProcessUUIDs.isEmpty()) {
        count = EnvTool.getAllQueriers(queryList).getNumberOfProcessInstances(visibleProcessUUIDs);
      }
    } else {
      count = EnvTool.getAllQueriers(queryList).getNumberOfProcessInstances();
    }
	  return count;
	}

	public ProcessInstance getProcessInstance(final ProcessInstanceUUID instanceUUID,  final String queryList) throws InstanceNotFoundException {
		final ProcessInstance result = getInternalProcessInstance(instanceUUID, queryList);  
		if (result == null) {
			throw new InstanceNotFoundException("bai_QRAPII_1", instanceUUID);
		}
		return new ProcessInstanceImpl(result);
	}

	public LightProcessInstance getLightProcessInstance(ProcessInstanceUUID instanceUUID, String queryList) throws InstanceNotFoundException, RemoteException {
		final ProcessInstance result = getInternalProcessInstance(instanceUUID, queryList);  
		if (result == null) {
			throw new InstanceNotFoundException("bai_QRAPII_1", instanceUUID);
		}
		return new LightProcessInstanceImpl(result);
	}

	ProcessInstance getInternalProcessInstance(final ProcessInstanceUUID instanceUUID, 
			final String queryList) throws InstanceNotFoundException {
		FacadeUtil.checkArgsNotNull(instanceUUID);
		Querier querier = EnvTool.getAllQueriers(queryList);
		final ProcessInstance result = querier.getProcessInstance(instanceUUID);
		return result;
	}

	public Set<ProcessInstance> getProcessInstances(final String queryList) {
    Set<InternalProcessInstance> processes = getInternalProcessInstances(queryList);

    final Set<ProcessInstance> result = new HashSet<ProcessInstance>();
    for (final ProcessInstance record : processes) {
      result.add(new ProcessInstanceImpl(record));
    }
    return result;
	}

	public Set<LightProcessInstance> getLightProcessInstances(String queryList) throws RemoteException {
	  Set<InternalProcessInstance> processes = getInternalProcessInstances(queryList);

		final Set<LightProcessInstance> result = new HashSet<LightProcessInstance>();
		for (final ProcessInstance record : processes) {
			result.add(new LightProcessInstanceImpl(record));
		}
		return result;
	}

	private Set<InternalProcessInstance> getInternalProcessInstances(String queryList) {
	  Environment env = Environment.getCurrent();
    boolean access = env.isRestrictedApplicationAcces();
    Set<InternalProcessInstance> processes = new HashSet<InternalProcessInstance>();
    if (access) {
      String applicationName = env.getApplicationAccessName();
      if(applicationName!=null) {
        Set<ProcessDefinitionUUID> visibleProcessUUIDs = FacadeUtil.getAllowedProcessUUIDsFor(applicationName, RuleType.PROCESS_READ);
        if (visibleProcessUUIDs != null && !visibleProcessUUIDs.isEmpty()) {
          processes = EnvTool.getAllQueriers(queryList).getProcessInstances(visibleProcessUUIDs);
        }
      }
    } else {
      processes = EnvTool.getAllQueriers(queryList).getProcessInstances();
    }
    return processes;
	}

	public List<LightProcessInstance> getLightProcessInstances(int fromIndex, int pageSize, String queryList) throws RemoteException {
    final List<LightProcessInstance> result = new ArrayList<LightProcessInstance>();
    if (pageSize <= 0) {
      return result;
    }

    Environment env = Environment.getCurrent();
    boolean access = env.isRestrictedApplicationAcces();
    List<InternalProcessInstance> processes = new ArrayList<InternalProcessInstance>();
    if (access) {
      String applicationName = env.getApplicationAccessName();
      if(applicationName!=null) {
        Set<ProcessDefinitionUUID> visibleProcessUUIDs = FacadeUtil.getAllowedProcessUUIDsFor(applicationName, RuleType.PROCESS_READ);
        if (visibleProcessUUIDs != null && !visibleProcessUUIDs.isEmpty()) {
          processes = EnvTool.getAllQueriers(queryList).getProcessInstances(visibleProcessUUIDs, fromIndex, pageSize);
        }
      }
    } else {
      processes = EnvTool.getAllQueriers(queryList).getProcessInstances(fromIndex, pageSize);
    }    

    for (final ProcessInstance record : processes) {
      result.add(new LightProcessInstanceImpl(record));
    }
    return result;
  }

	public List<LightProcessInstance> getLightParentProcessInstances(int fromIndex, int pageSize, String queryList) throws RemoteException {
	  final List<LightProcessInstance> result = new ArrayList<LightProcessInstance>();
    if (pageSize <= 0) {
      return result;
    }
    for (final ProcessInstance record : EnvTool.getAllQueriers(queryList).getParentProcessInstances(fromIndex ,pageSize)) {
      result.add(new LightProcessInstanceImpl(record));
    }
    return result;
	}

	public Set<ProcessInstance> getUserInstances(String queryList) throws RemoteException {
	  Set<InternalProcessInstance> processes = getUserProcessInstances(queryList);
		final Set<ProcessInstance> result = new HashSet<ProcessInstance>();
		for (final ProcessInstance record : processes) {
			result.add(new ProcessInstanceImpl(record));
		}
		return result;
	}

	public Set<LightProcessInstance> getLightUserInstances(String queryList) throws RemoteException {
	  Set<InternalProcessInstance> processes = getUserProcessInstances(queryList);
		final Set<LightProcessInstance> result = new HashSet<LightProcessInstance>();
		for (final ProcessInstance record : processes) {
			result.add(new LightProcessInstanceImpl(record));
		}
		return result;
	}

	private Set<InternalProcessInstance> getUserProcessInstances(String queryList) {
    Environment env = Environment.getCurrent();
    boolean access = env.isRestrictedApplicationAcces();
    Set<InternalProcessInstance> processes = new HashSet<InternalProcessInstance>();
    if (access) {
      String applicationName = env.getApplicationAccessName();
      if(applicationName!=null) {
        Set<ProcessDefinitionUUID> visibleProcessUUIDs = FacadeUtil.getAllowedProcessUUIDsFor(applicationName, RuleType.PROCESS_READ);
        if (visibleProcessUUIDs != null && !visibleProcessUUIDs.isEmpty()) {
          processes = EnvTool.getAllQueriers(queryList).getUserInstances(EnvTool.getUserId(), visibleProcessUUIDs);
        }
      }
    } else {
      processes = EnvTool.getAllQueriers(queryList).getUserInstances(EnvTool.getUserId());
    }
    return processes;
  }

	public Set<ProcessInstance> getProcessInstances(Collection<ProcessInstanceUUID> instanceUUIDs, String queryList) throws RemoteException {
		final Set<ProcessInstance> result = new HashSet<ProcessInstance>();
		if (instanceUUIDs == null || instanceUUIDs.isEmpty()) {
			return result;
		}
		for (final ProcessInstance record : EnvTool.getAllQueriers(queryList).getProcessInstances(instanceUUIDs)) {
			result.add(new ProcessInstanceImpl(record));
		}
		return result;
	}

	public Set<LightProcessInstance> getLightProcessInstances(Collection<ProcessInstanceUUID> instanceUUIDs, String queryList) throws RemoteException {
		final Set<LightProcessInstance> result = new HashSet<LightProcessInstance>();
		if (instanceUUIDs == null || instanceUUIDs.isEmpty()) {
			return result;
		}
		for (final ProcessInstance record : EnvTool.getAllQueriers(queryList).getProcessInstances(instanceUUIDs)) {
			result.add(new LightProcessInstanceImpl(record));
		}
		return result;
	}
	
	public List<LightProcessInstance> getLightProcessInstances(Collection<ProcessInstanceUUID> instanceUUIDs, int fromIndex, int pageSize, String queryList) 
	throws RemoteException {
	  final List<LightProcessInstance> result = new ArrayList<LightProcessInstance>();
	  if (pageSize <= 0) {
      return result;
    }
    if (instanceUUIDs == null || instanceUUIDs.isEmpty()) {
      return result;
    }
    for (final ProcessInstance record : EnvTool.getAllQueriers(queryList).getProcessInstances(instanceUUIDs, fromIndex, pageSize)) {
      result.add(new LightProcessInstanceImpl(record));
    }
    return result;
	}

	public Set<ProcessInstance> getProcessInstancesWithTaskState(Collection<ActivityState> activityStates, String queryList) throws RemoteException {
		final Set<ProcessInstance> result = new HashSet<ProcessInstance>();
		if (activityStates == null || activityStates.isEmpty()) {
			return result;
		}

		Environment env = Environment.getCurrent();
    boolean access = env.isRestrictedApplicationAcces();
    Set<InternalProcessInstance> processes = new HashSet<InternalProcessInstance>();
    if (access) {
      String applicationName = env.getApplicationAccessName();
      if(applicationName!=null) {
        Set<ProcessDefinitionUUID> visibleProcessUUIDs = FacadeUtil.getAllowedProcessUUIDsFor(applicationName, RuleType.PROCESS_READ);
        if (visibleProcessUUIDs != null && !visibleProcessUUIDs.isEmpty()) {
          processes = EnvTool.getAllQueriers(queryList).getProcessInstancesWithTaskState(activityStates, visibleProcessUUIDs);
        }
      }
    } else {
      processes = EnvTool.getAllQueriers(queryList).getProcessInstancesWithTaskState(activityStates);
    }

		for (final ProcessInstance record : processes) {
			result.add(new ProcessInstanceImpl(record));
		}
		return result;
	}
	
  public Set<ProcessInstance> getProcessInstancesWithInstanceStates(Collection<InstanceState> instanceStates,
      String queryList) throws RemoteException {
    Misc.checkArgsNotNull(instanceStates);
    if (instanceStates.isEmpty()) {
      throw new IllegalArgumentException(ExceptionManager.getInstance().getMessage("bai_QRAPII_15"));
    }
    final Set<ProcessInstance> result = new HashSet<ProcessInstance>();

    Environment env = Environment.getCurrent();
    boolean access = env.isRestrictedApplicationAcces();
    Set<InternalProcessInstance> processes = new HashSet<InternalProcessInstance>();
    if (access) {
      String applicationName = env.getApplicationAccessName();
      if (applicationName != null) {
        Set<ProcessDefinitionUUID> visibleProcessUUIDs = FacadeUtil.getAllowedProcessUUIDsFor(applicationName,
            RuleType.PROCESS_READ);
        if (visibleProcessUUIDs != null && !visibleProcessUUIDs.isEmpty()) {
          processes = EnvTool.getAllQueriers(queryList).getProcessInstancesWithInstanceStates(instanceStates,
              visibleProcessUUIDs);
        }
      }
    } else {
      processes = EnvTool.getAllQueriers(queryList).getProcessInstancesWithInstanceStates(instanceStates);
    }

    for (final ProcessInstance record : processes) {
      result.add(new ProcessInstanceImpl(record));
    }
    return result;
  }

	public Set<ProcessInstance> getProcessInstances(final ProcessDefinitionUUID processUUID, final String queryList) {
		final Set<ProcessInstance> result = new HashSet<ProcessInstance>();
		for (final ProcessInstance record : EnvTool.getAllQueriers(queryList).getProcessInstances(processUUID)) {
			result.add(new ProcessInstanceImpl(record));
		}
		return result;
	}

	public Set<LightProcessInstance> getLightProcessInstances(ProcessDefinitionUUID processUUID, String queryList) throws RemoteException {
	  final Set<LightProcessInstance> result = new HashSet<LightProcessInstance>();
    for (final ProcessInstance record : EnvTool.getAllQueriers(queryList).getProcessInstances(processUUID)) {
      result.add(new LightProcessInstanceImpl(record));
    }
    return result;
	}

	public Set<LightProcessInstance> getLightWeightProcessInstances(Set<ProcessDefinitionUUID> processUUIDs, String queryList) throws RemoteException {
		final Set<LightProcessInstance> result = new HashSet<LightProcessInstance>();
		for (final ProcessInstance record : EnvTool.getAllQueriers(queryList).getProcessInstances(processUUIDs)) {
			result.add(new LightProcessInstanceImpl(record));
		}
		return result;
	}

	public ActivityInstance getActivityInstance(final ProcessInstanceUUID instanceUUID, final String activityId,
			final String iterationId, final String activityInstanceId, 
			final String queryList) throws ActivityNotFoundException, InstanceNotFoundException {
		final ActivityInstance result =
			EnvTool.getAllQueriers(queryList).getActivityInstance(instanceUUID, activityId, iterationId, activityInstanceId);
		if (result == null) {
			if (EnvTool.getAllQueriers(queryList).getProcessInstance(instanceUUID) == null) {
				throw new InstanceNotFoundException("bai_QRAPII_2", instanceUUID);
			}
			throw new ActivityNotFoundException("bai_QRAPII_3", instanceUUID, activityId, iterationId);
		}
		return new ActivityInstanceImpl(result);
	}

	public boolean canExecuteTask(ActivityInstanceUUID taskUUID, String queryList) throws TaskNotFoundException, RemoteException {
		TaskInstance task = getTask(taskUUID, queryList);
		if (task == null) {
			return false;
		}
		if (!task.getState().equals(ActivityState.READY)) {
			return false;
		}
		String userId = EnvTool.getUserId();
		if (task.isTaskAssigned()) {
			return task.getTaskUser().equals(userId);
		}
		return task.getTaskCandidates().contains(userId);

	}

	public Set<ActivityInstance> getActivityInstances(final ProcessInstanceUUID instanceUUID, final String queryList) {
		final Set<ActivityInstance> result = new HashSet<ActivityInstance>();
		for (final ActivityInstance record : EnvTool.getAllQueriers(queryList).getActivityInstances(instanceUUID)) {
			result.add(new ActivityInstanceImpl(record));
		}
		return result;
	}

	public List<LightActivityInstance> getLightActivityInstancesFromRoot(ProcessInstanceUUID rootInstanceUUID, String queryList) throws RemoteException {
	  final List<LightActivityInstance> result = new ArrayList<LightActivityInstance>();
	  List<InternalActivityInstance> activities = EnvTool.getAllQueriers(queryList).getActivityInstancesFromRoot(rootInstanceUUID);
	  
    for (final InternalActivityInstance record : activities) {
      result.add(new LightActivityInstanceImpl(record));
    }
    return result;
	}

	public Map<ProcessInstanceUUID, List<LightActivityInstance>> getLightActivityInstancesFromRoot(Set<ProcessInstanceUUID> rootInstanceUUIDs, ActivityState state, String queryList) throws RemoteException {
	  Map<ProcessInstanceUUID, List<LightActivityInstance>> result = new HashMap<ProcessInstanceUUID, List<LightActivityInstance>>();
    List<InternalActivityInstance> activities = EnvTool.getAllQueriers(queryList).getActivityInstancesFromRoot(rootInstanceUUIDs, state);
    for (InternalActivityInstance activity : activities) {
      final ProcessInstanceUUID instanceUUID = activity.getRootInstanceUUID();
      if (!result.containsKey(instanceUUID)) {
        result.put(instanceUUID, new ArrayList<LightActivityInstance>());
      }
      result.get(instanceUUID).add(new LightActivityInstanceImpl(activity));
    }
    return result;
	}
	
	
	public Map<ProcessInstanceUUID, List<LightActivityInstance>> getLightActivityInstancesFromRoot(Set<ProcessInstanceUUID> rootInstanceUUIDs, String queryList) throws RemoteException {
	  Map<ProcessInstanceUUID, List<LightActivityInstance>> result = new HashMap<ProcessInstanceUUID, List<LightActivityInstance>>();
	  List<InternalActivityInstance> activities = EnvTool.getAllQueriers(queryList).getActivityInstancesFromRoot(rootInstanceUUIDs);
	  for (InternalActivityInstance activity : activities) {
	    final ProcessInstanceUUID instanceUUID = activity.getRootInstanceUUID();
      if (!result.containsKey(instanceUUID)) {
        result.put(instanceUUID, new ArrayList<LightActivityInstance>());
      }
      result.get(instanceUUID).add(new LightActivityInstanceImpl(activity));
    }
	  return result;
	}
	
	public Map<ProcessInstanceUUID, LightActivityInstance> getLightLastUpdatedActivityInstanceFromRoot(Set<ProcessInstanceUUID> rootInstanceUUIDs, boolean considerSystemTaks, String queryList) throws RemoteException {
	  Map<ProcessInstanceUUID, LightActivityInstance> result = new HashMap<ProcessInstanceUUID, LightActivityInstance>();
	  Map<ProcessInstanceUUID, InternalActivityInstance> temp = EnvTool.getAllQueriers(queryList).getLastUpdatedActivityInstanceFromRoot(rootInstanceUUIDs, considerSystemTaks);
	  for (Map.Entry<ProcessInstanceUUID, InternalActivityInstance> entry : temp.entrySet()) {
	    result.put(entry.getKey(), new LightActivityInstanceImpl(entry.getValue()));
    }
    return result;
	}

	public List<LightTaskInstance> getLightTaskInstancesFromRoot(ProcessInstanceUUID rootInstanceUUID, String queryList) throws RemoteException {
	  final List<LightTaskInstance> result = new ArrayList<LightTaskInstance>();
    for (final InternalActivityInstance record : EnvTool.getAllQueriers(queryList).getActivityInstancesFromRoot(rootInstanceUUID)) {
      if (record.isTask()) {
        result.add(new LightActivityInstanceImpl(record));
      }
    }
    return result;
	}

	public Map<ProcessInstanceUUID, List<LightTaskInstance>> getLightTaskInstancesFromRoot(Set<ProcessInstanceUUID> rootInstanceUUIDs, String queryList)
	throws RemoteException {
	  Map<ProcessInstanceUUID, List<LightTaskInstance>> result = new HashMap<ProcessInstanceUUID, List<LightTaskInstance>>();
    for (ProcessInstanceUUID instanceUUID : rootInstanceUUIDs) {
      result.put(instanceUUID, getLightTaskInstancesFromRoot(instanceUUID, queryList));
    }
    return result;
	}

	public Set<LightActivityInstance> getLightActivityInstances(ProcessInstanceUUID instanceUUID, String queryList)
	throws InstanceNotFoundException, RemoteException {
		final Set<LightActivityInstance> result = new HashSet<LightActivityInstance>();
		final Set<InternalActivityInstance> activities = EnvTool.getAllQueriers(queryList).getActivityInstances(instanceUUID); 
		for (final ActivityInstance record : activities) {
			result.add(new LightActivityInstanceImpl(record));
		}
		return result;
	}

	public Set<ActivityInstance> getActivityInstances(final ProcessInstanceUUID instanceUUID, final String activityId, final String queryList)
	throws ActivityNotFoundException {
		final Set<ActivityInstance> result = new HashSet<ActivityInstance>();
		for (final ActivityInstance record 
				: EnvTool.getAllQueriers(queryList).getActivityInstances(instanceUUID)) {
			if (record.getActivityName().equals(activityId)) {
				result.add(new ActivityInstanceImpl(record));
			}
		}
		if (result.isEmpty()) {
			throw new ActivityNotFoundException("bai_QRAPII_4", instanceUUID, activityId);
		}
		return result;
	}

	public Set<LightActivityInstance> getLightActivityInstances(ProcessInstanceUUID instanceUUID, String activityName, String queryList)
  throws InstanceNotFoundException, ActivityNotFoundException, RemoteException {
	  final Set<LightActivityInstance> result = new HashSet<LightActivityInstance>();
	  Set<InternalActivityInstance> activities = EnvTool.getAllQueriers(queryList).getActivityInstances(instanceUUID);
	  for (final InternalActivityInstance record : activities) {
      if (record.getActivityName().equals(activityName)) {
        result.add(new LightActivityInstanceImpl(record));
      }
    }
    if (result.isEmpty()) {
      throw new ActivityNotFoundException("bai_QRAPII_4", instanceUUID, activityName);
    }
    return result;
  }

	public Set<TaskInstance> getTasks(final ProcessInstanceUUID instanceUUID, final String queryList) {
		final Set<TaskInstance> result = new HashSet<TaskInstance>();
		for (final TaskInstance record : EnvTool.getAllQueriers(queryList).getTaskInstances(instanceUUID)) {
			result.add(new ActivityInstanceImpl(record));
		}
		return result;
	}
	
	public Set<LightTaskInstance> getLightTasks(ProcessInstanceUUID instanceUUID, String queryList) throws InstanceNotFoundException, RemoteException {
		final Set<LightTaskInstance> result = new HashSet<LightTaskInstance>();
		for (final TaskInstance record : EnvTool.getAllQueriers(queryList).getTaskInstances(instanceUUID)) {
			result.add(new LightActivityInstanceImpl(record));
		}
		return result;
	}

	public Collection<TaskInstance> getTaskList(final ProcessInstanceUUID instanceUUID, final ActivityState taskState, final String queryList) throws InstanceNotFoundException {
		return getTaskListUser(instanceUUID, EnvTool.getUserId(), taskState, queryList);
	}

	public Collection<LightTaskInstance> getLightTaskList(final ProcessInstanceUUID instanceUUID, final ActivityState taskState, final String queryList) throws InstanceNotFoundException {
    return getLightTaskListUser(instanceUUID, EnvTool.getUserId(), taskState, queryList);
  }
	
	public Collection<TaskInstance> getTaskList(ProcessInstanceUUID instanceUUID, Collection<ActivityState> taskStates, String queryList) throws InstanceNotFoundException, RemoteException {
		FacadeUtil.checkArgsNotNull(instanceUUID, taskStates);
		final Collection<TaskInstance> todos = new HashSet<TaskInstance>();
		for (ActivityState taskState : taskStates) {
			Collection<TaskInstance> tasks = getTaskListUser(instanceUUID, EnvTool.getUserId(), taskState, queryList);
			if (tasks != null) {
				todos.addAll(tasks);
			}
		}
		return todos;
	}

	public Collection<LightTaskInstance> getLightTaskList(ProcessInstanceUUID instanceUUID, Collection<ActivityState> taskStates, String queryList) throws InstanceNotFoundException, RemoteException {
    FacadeUtil.checkArgsNotNull(instanceUUID, taskStates);
    final Collection<LightTaskInstance> todos = new HashSet<LightTaskInstance>();
    for (ActivityState taskState : taskStates) {
      Collection<LightTaskInstance> tasks = getLightTaskListUser(instanceUUID, EnvTool.getUserId(), taskState, queryList);
      if (tasks != null) {
        todos.addAll(tasks);
      }
    }
    return todos;
  }
	
	public Collection<TaskInstance> getTaskList(final ActivityState taskState, final String queryList) {
		return getTaskListUser(EnvTool.getUserId(), taskState, queryList);
	}

	public Collection<LightTaskInstance> getLightTaskList(final ActivityState taskState, final String queryList) {
    return getLightTaskListUser(EnvTool.getUserId(), taskState, queryList);
  }
	
	public TaskInstance getTask(final ActivityInstanceUUID taskUUID, final String queryList) throws TaskNotFoundException {
	  final TaskInstance taskInstance = EnvTool.getAllQueriers(queryList).getTaskInstance(taskUUID);
		if (taskInstance == null) {
			throw new TaskNotFoundException("bai_QRAPII_5", taskUUID);
		}
		return new ActivityInstanceImpl(taskInstance);
	}
	
	public Set<String> getTaskCandidates(ActivityInstanceUUID taskUUID, String queryList) throws RemoteException, TaskNotFoundException {
	  final TaskInstance taskInstance = EnvTool.getAllQueriers(queryList).getTaskInstance(taskUUID);
    if (taskInstance == null) {
      throw new TaskNotFoundException("bai_QRAPII_5", taskUUID);
    }
    return CopyTool.copy(taskInstance.getTaskCandidates());
	}
	
	public Map<ActivityInstanceUUID, Set<String>> getTaskCandidates(Set<ActivityInstanceUUID> taskUUIDs, String queryList) throws RemoteException, TaskNotFoundException {
	  final Map<ActivityInstanceUUID, Set<String>> result = new HashMap<ActivityInstanceUUID, Set<String>>();
	  for (ActivityInstanceUUID taskUUID : taskUUIDs) {
	    final TaskInstance taskInstance = EnvTool.getAllQueriers(queryList).getTaskInstance(taskUUID);
	    if (taskInstance == null) {
	      throw new TaskNotFoundException("bai_QRAPII_5", taskUUID);
	    }
	    result.put(taskUUID, CopyTool.copy(taskInstance.getTaskCandidates()));  
    }
	  return result;
	}

	public Map<String, Object> getActivityInstanceVariables(final ProcessInstanceUUID instanceUUID,
			final String activityId, final String iterationId, final String activityInstanceId, 
			final String queryList) throws ActivityNotFoundException, InstanceNotFoundException {

		final ActivityInstance activityInst = getActivityInstance(instanceUUID, activityId, iterationId, activityInstanceId, queryList);
		if (activityInst == null) {
			throw new ActivityNotFoundException("bai_QRAPII_6", instanceUUID, activityId);
		}
		Map<String, Object> variables = activityInst.getLastKnownVariableValues();
		/*Map<String, Object> convertedVariables = new HashMap<String, Object>();
    for (Map.Entry<String, Object> variable : variables.entrySet()) {
      Object value = variable.getValue();
      if (value != null && (value.getClass().getClassLoader() instanceof ProcessClassLoader
      || value.getClass().getClassLoader() instanceof CommonClassLoader)) {
        convertedVariables.put(variable.getKey(), new ObjectVariable((Serializable)value));
      } else {
        convertedVariables.put(variable.getKey(), value);
      }
    }
    return convertedVariables;
		 */
		return variables;
	}

	public Object getActivityInstanceVariable(final ProcessInstanceUUID instanceUUID,
			final String activityId, final String iterationId,
			final String activityInstanceId, final String variableId, final String queryList) throws InstanceNotFoundException,
			ActivityNotFoundException, VariableNotFoundException {

		final Map<String, Object> variables = getActivityInstanceVariables(instanceUUID, activityId, iterationId, activityInstanceId, queryList);
		if (variables == null || !variables.containsKey(variableId)) {
			throw new VariableNotFoundException("bai_QRAPII_7", instanceUUID, activityId, variableId);
		}
		return variables.get(variableId);
	}

	public Map<String, Object> getProcessInstanceVariables(
			final ProcessInstanceUUID instanceUUID, final String queryList) throws InstanceNotFoundException {
		final ProcessInstance processInstance = getInternalProcessInstance(instanceUUID, queryList);
		if (processInstance == null) {
			throw new InstanceNotFoundException("bai_QRAPII_8", instanceUUID);
		}
		return processInstance.getLastKnownVariableValues();
	}

	private Map<String, Object> getProcessInstanceVariables(
			final ProcessInstanceUUID instanceUUID, final String queryList, final Date maxDate) throws InstanceNotFoundException {
		//take all initial instance var and for each varupdate being proceed before max date, replace the initial value by the new one  
		final ProcessInstance processInstance = getInternalProcessInstance(instanceUUID, queryList);

		final Map<String, Object> instanceInitialVars = processInstance.getInitialVariableValues();
		final Map<String, Object> instanceVarBeforeMaxDate = new HashMap<String, Object>();
		instanceVarBeforeMaxDate.putAll(instanceInitialVars);

		final Map<String, VariableUpdate> maxVarUpdates = new HashMap<String, VariableUpdate>();

		for (VariableUpdate varUpdate : processInstance.getVariableUpdates()) {
			if (varUpdate.getDate().getTime() <= maxDate.getTime()) {
				VariableUpdate currentMax = maxVarUpdates.get(varUpdate.getName()); 
				if (currentMax == null || currentMax.getDate().getTime() <= varUpdate.getDate().getTime()) {
					maxVarUpdates.put(varUpdate.getName(), varUpdate);
					instanceVarBeforeMaxDate.put(varUpdate.getName(), varUpdate.getValue());
				}
			}
		}
		return instanceVarBeforeMaxDate;
	}

	private Object getProcessInstanceVariable(final ProcessInstanceUUID instanceUUID,
			final String variableId, final String queryList, 
			final Date maxDate) throws InstanceNotFoundException, VariableNotFoundException {
		final Map<String, Object> variables = getProcessInstanceVariables(instanceUUID, queryList, maxDate);
		if (variables == null || !variables.containsKey(variableId)) {
			throw new VariableNotFoundException("bai_QRAPII_9", instanceUUID, variableId);
		}
		return variables.get(variableId);
	}

	public Object getProcessInstanceVariable(final ProcessInstanceUUID instanceUUID,
			final String variableId, final String queryList) throws InstanceNotFoundException, VariableNotFoundException {
		final Map<String, Object> variables = getProcessInstanceVariables(instanceUUID, queryList);
		if (variables == null || !variables.containsKey(variableId)) {
			throw new VariableNotFoundException("bai_QRAPII_10", instanceUUID, variableId);
		}
		return variables.get(variableId);
	}

	public ActivityInstance getActivityInstance(final ActivityInstanceUUID activityUUID, final String queryList) throws ActivityNotFoundException {
		final ActivityInstance activity = EnvTool.getAllQueriers(queryList).getActivityInstance(activityUUID);
		if (activity == null) {
			throw new ActivityNotFoundException("bai_QRAPII_11", activityUUID);
		}
		return new ActivityInstanceImpl(activity);
	}

	public ActivityState getActivityInstanceState(final ActivityInstanceUUID activityUUID,
			final String queryList) throws ActivityNotFoundException {
		FacadeUtil.checkArgsNotNull(activityUUID);
		final Querier querier = EnvTool.getAllQueriers(queryList);
		ActivityState state = querier.getActivityInstanceState(activityUUID);
		if (state == null) {
			throw new ActivityNotFoundException("bai_QRAPII_3", activityUUID.getProcessInstanceUUID(), activityUUID.getActivityName());
		}
		return state;
	}

	public Object getActivityInstanceVariable(final ActivityInstanceUUID activityUUID,
			final String variableId, final String queryList) throws ActivityNotFoundException,
			VariableNotFoundException {
		final ActivityInstance activity = getActivityInstance(activityUUID, queryList);
		final Map<String, Object> variableValues = activity.getLastKnownVariableValues();

		if (!variableValues.containsKey(variableId)) {
			throw new VariableNotFoundException("bai_QRAPII_12", activityUUID, variableId);
		}
		return activity.getLastKnownVariableValues().get(variableId);
	}

	public Map<String, Object> getActivityInstanceVariables(
			final ActivityInstanceUUID activityUUID, final String queryList) throws ActivityNotFoundException {
		final ActivityInstance activity = getActivityInstance(activityUUID, queryList);
		return activity.getLastKnownVariableValues();
	}

	public Collection<TaskInstance> getTaskList(final ProcessInstanceUUID instanceUUID,
			final String userId, final ActivityState taskState, final String queryList) throws InstanceNotFoundException {
		return getTaskListUser(instanceUUID, userId, taskState, queryList);
	}

	public Collection<LightTaskInstance> getLightTaskList(final ProcessInstanceUUID instanceUUID,
      final String userId, final ActivityState taskState, final String queryList) throws InstanceNotFoundException {
    return getLightTaskListUser(instanceUUID, userId, taskState, queryList);
  }
	
	public Collection<TaskInstance> getTaskList(final String userId, final ActivityState taskState, final String queryList) {
		return getTaskListUser(userId, taskState, queryList);
	}

	public Collection<LightTaskInstance> getLightTaskList(final String userId, final ActivityState taskState, final String queryList) {
    return getLightTaskListUser(userId, taskState, queryList);
  }
	
	public ActivityInstanceUUID getOneTask(ActivityState taskState, String queryList) throws RemoteException {
	  final Querier journal = EnvTool.getJournalQueriers(queryList);

	  Environment env = Environment.getCurrent();
	  boolean access = env.isRestrictedApplicationAcces();
	  TaskInstance task = null;
	  if (access) {
	    String applicationName = env.getApplicationAccessName();
	    if(applicationName!=null) {
	      Set<ProcessDefinitionUUID> visibleProcessUUIDs = FacadeUtil.getAllowedProcessUUIDsFor(applicationName, RuleType.PROCESS_READ);
	      if (visibleProcessUUIDs != null && !visibleProcessUUIDs.isEmpty()) {
	        task = journal.getOneTask(EnvTool.getUserId(), taskState, visibleProcessUUIDs);
	      }
	    }
	  } else {
	    task = journal.getOneTask(EnvTool.getUserId(), taskState);
	  }
	  if (task == null) {
      return null;
    }
    return new ActivityInstanceUUID(task.getUUID());
	}

	public ActivityInstanceUUID getOneTask(ProcessInstanceUUID instanceUUID, ActivityState taskState, String queryList) throws RemoteException {
	  final Querier journal = EnvTool.getJournalQueriers(queryList);
	  TaskInstance task = journal.getOneTask(EnvTool.getUserId(), instanceUUID, taskState);
	  if (task == null) {
	    return null;
	  }
	  return new ActivityInstanceUUID(task.getUUID());
	}
	
	public ActivityInstanceUUID getOneTask(ProcessDefinitionUUID processUUID, ActivityState taskState, String queryList) throws RemoteException {
	  final Querier journal = EnvTool.getJournalQueriers(queryList);
    TaskInstance task = journal.getOneTask(EnvTool.getUserId(), processUUID, taskState);
    if (task == null) {
      return null;
    }
    return new ActivityInstanceUUID(task.getUUID());
	}
	
	private Collection<TaskInstance> getTaskListUser(final ProcessInstanceUUID instanceUUID, final String userId, final ActivityState taskState, final String queryList) throws InstanceNotFoundException {
	  final Collection<TaskInstance> todos = new ArrayList<TaskInstance>();
    for (final TaskInstance taskActivity : getInternalTaskListUser(instanceUUID, userId, taskState, queryList)) {
      todos.add(new ActivityInstanceImpl(taskActivity));
    }
    return todos;
	}
	
	private Collection<LightTaskInstance> getLightTaskListUser(final ProcessInstanceUUID instanceUUID, final String userId, final ActivityState taskState, final String queryList) throws InstanceNotFoundException {
	  final Collection<LightTaskInstance> todos = new ArrayList<LightTaskInstance>();
    for (final TaskInstance taskActivity : getInternalTaskListUser(instanceUUID, userId, taskState, queryList)) {
      todos.add(new LightActivityInstanceImpl(taskActivity));
    }
    return todos;
	}
	
	private Collection<TaskInstance> getInternalTaskListUser(final ProcessInstanceUUID instanceUUID, final String userId, final ActivityState taskState, final String queryList) throws InstanceNotFoundException {
		FacadeUtil.checkArgsNotNull(instanceUUID, taskState, userId);
		final ProcessInstance processInstance = getInternalProcessInstance(instanceUUID, queryList);
		if (processInstance == null) {
			throw new InstanceNotFoundException("bai_QRAPII_13", instanceUUID);
		}
		final Querier journal = EnvTool.getAllQueriers(queryList);
		return journal.getUserInstanceTasks(userId, instanceUUID, taskState);
	}

	private Collection<TaskInstance> getTaskListUser(final String userId, final ActivityState taskState, final String queryList) {
	  final Collection<TaskInstance> result = new HashSet<TaskInstance>();
    for (TaskInstance taskInstance : getInternalTaskListUser(userId, taskState, queryList)) {
      result.add(new ActivityInstanceImpl(taskInstance));
    }
    return result;
	}
	
	private Collection<LightTaskInstance> getLightTaskListUser(final String userId, final ActivityState taskState, final String queryList) {
	  final Collection<LightTaskInstance> result = new HashSet<LightTaskInstance>();
    for (TaskInstance taskInstance : getInternalTaskListUser(userId, taskState, queryList)) {
      result.add(new LightActivityInstanceImpl(taskInstance));
    }
    return result;
	}
	
	private Collection<TaskInstance> getInternalTaskListUser(final String userId, final ActivityState taskState, final String queryList) {
		FacadeUtil.checkArgsNotNull(userId, taskState);
	
		Environment env = Environment.getCurrent();
    boolean access = env.isRestrictedApplicationAcces();
    Collection<TaskInstance> tasks = new HashSet<TaskInstance>();
    if (access) {
      String applicationName = env.getApplicationAccessName();
      if(applicationName!=null) {
        Set<ProcessDefinitionUUID> visibleProcessUUIDs = FacadeUtil.getAllowedProcessUUIDsFor(applicationName, RuleType.PROCESS_READ);
        if (visibleProcessUUIDs != null && !visibleProcessUUIDs.isEmpty()) {
          tasks = EnvTool.getAllQueriers(queryList).getUserTasks(userId, taskState, visibleProcessUUIDs);
        }
      }
    } else {
      tasks = EnvTool.getAllQueriers(queryList).getUserTasks(userId, taskState);
    }
    return tasks;
	}

	public Object getVariable(final ActivityInstanceUUID activityUUID, final String variableId, 
			final String queryList) throws ActivityNotFoundException, VariableNotFoundException {
		try {
			return getActivityInstanceVariable(activityUUID, variableId, queryList);
		} catch (final VariableNotFoundException e) {
			final ActivityInstance activity = getActivityInstance(activityUUID, queryList);
			Date maxDate = getMaxDate(activity);

			try {
				return getProcessInstanceVariable(activity.getProcessInstanceUUID(), variableId, queryList, maxDate);
			} catch (final InstanceNotFoundException e1) {
				// If activity exists, the process instance must exist too.
				Misc.unreachableStatement();
				return null;
			}
		}
	}

	public Map<String, Object> getVariables(final ActivityInstanceUUID activityUUID, 
			final String queryList) throws ActivityNotFoundException {
		final ActivityInstance activity = EnvTool.getAllQueriers(queryList).getActivityInstance(activityUUID);
		if (activity == null) {
			throw new ActivityNotFoundException("bai_QRAPII_14", activityUUID);
		}
		Date maxDate = getMaxDate(activity);
		try {
			final Map<String, Object> allVariables = new HashMap<String, Object>();
			final Map<String, Object> localVariables = activity.getLastKnownVariableValues();
			final Map<String, Object> globalVariables = 
				getProcessInstanceVariables(activity.getProcessInstanceUUID(), queryList, maxDate);
			//add global first because if some variables are in both local and global
			//we want to keep local value
			allVariables.putAll(globalVariables);
			allVariables.putAll(localVariables);
			return allVariables;
		} catch (final InstanceNotFoundException e) {
			// If activity exists, the process instance must exist too.
			Misc.unreachableStatement();
			return null;
		}
	}

	private Date getMaxDate(ActivityInstance activity) {
		Date endedDate = activity.getEndedDate();
		if (endedDate == null) {
			return new Date();
		}
		return endedDate;

	}

	public Set<String> getAttachmentNames(ProcessInstanceUUID instanceUUID, String queryList) throws RemoteException {
		FacadeUtil.checkArgsNotNull(instanceUUID);
		Set<String> result = new HashSet<String>();
		final InternalProcessInstance instance =
			EnvTool.getAllQueriers(queryList).getProcessInstance(instanceUUID);
		List<AttachmentInstance> attachmentInstances = instance.getAttachments();
		for (AttachmentInstance attachmentInstance : attachmentInstances) {
			result.add(attachmentInstance.getName());
		}
		return result;
	}

	public AttachmentInstance getLastAttachment(ProcessInstanceUUID instanceUUID, String attachmentName, String queryList) throws RemoteException {
		FacadeUtil.checkArgsNotNull(instanceUUID, attachmentName);
		final InternalProcessInstance instance =
			EnvTool.getAllQueriers(queryList).getProcessInstance(instanceUUID);
		List<AttachmentInstance> matchingAttachments = instance.getAttachments(attachmentName);
		if (matchingAttachments.isEmpty()) {
			return null;
		} else {
			return new AttachmentInstanceImpl(matchingAttachments.get(matchingAttachments.size() - 1));
		}
	}

	public AttachmentInstance getLastAttachment(ProcessInstanceUUID instanceUUID, String attachmentName, ActivityInstanceUUID activityUUID, String queryList)
	throws RemoteException, ActivityNotFoundException {
		ActivityInstance activity = getActivityInstance(activityUUID, queryList);
		Date date = null;
		if (!activity.getState().equals(ActivityState.READY)
		    && !activity.getState().equals(ActivityState.SUSPENDED)
		    && !activity.getState().equals(ActivityState.EXECUTING)) {
			date = activity.getLastStateUpdate().getUpdatedDate();  
		} else {
			date = new Date();
		}
		return getLastAttachment(instanceUUID, attachmentName, date, queryList);
	}

	public AttachmentInstance getLastAttachment(ProcessInstanceUUID instanceUUID, String attachmentName, Date date, String queryList) throws RemoteException {
		FacadeUtil.checkArgsNotNull(instanceUUID, attachmentName, date);
		AttachmentInstance result = null;
		final InternalProcessInstance instance =
			EnvTool.getAllQueriers(queryList).getProcessInstance(instanceUUID);
		List<AttachmentInstance> matchingAttachments = instance.getAttachments(attachmentName);
		for (int i = 0; i < matchingAttachments.size(); i++) {
			if (result == null || matchingAttachments.get(i).getVersionDate().getTime() <= date.getTime()) {
				result = matchingAttachments.get(i);
			}
		}
		if (result == null) {
		  return null;
		}
		return new AttachmentInstanceImpl(result);
	}

	public Collection<AttachmentInstance> getLastAttachments(ProcessInstanceUUID instanceUUID, Set<String> attachmentNames, String queryList)
	throws RemoteException {
		FacadeUtil.checkArgsNotNull(instanceUUID, attachmentNames);
		Set<AttachmentInstance> result = new HashSet<AttachmentInstance>();
		for (String attachmentName : attachmentNames) {
			AttachmentInstance attachmentInstance = getLastAttachment(instanceUUID, attachmentName, queryList);
			if (attachmentInstance != null) {
				result.add(attachmentInstance);
			}
		}
		return result;
	}

	public Collection<AttachmentInstance> getLastAttachments(ProcessInstanceUUID instanceUUID, String regex, String queryList) throws RemoteException {
		FacadeUtil.checkArgsNotNull(instanceUUID, regex);
		final InternalProcessInstance instance =
			EnvTool.getAllQueriers(queryList).getProcessInstance(instanceUUID);
		List<AttachmentInstance> matchingAttachments = instance.getAttachments();
		Map<String, AttachmentInstance> result = new HashMap<String, AttachmentInstance>();
		for (AttachmentInstance attachmentInstance : matchingAttachments) {
			if (attachmentInstance.getName().matches(regex)) {
				result.put(attachmentInstance.getName(), new AttachmentInstanceImpl(attachmentInstance));
			}
		}
		return result.values();
	}

	public List<AttachmentInstance> getAttachments(ProcessInstanceUUID instanceUUID, String attachmentName, String queryList)
	throws RemoteException {
		FacadeUtil.checkArgsNotNull(instanceUUID, attachmentName);
		final InternalProcessInstance instance =
			EnvTool.getAllQueriers(queryList).getProcessInstance(instanceUUID);
		List<AttachmentInstance> attachments = new ArrayList<AttachmentInstance>();
		for (AttachmentInstance attachmentInstance : instance.getAttachments(attachmentName)) {
		  attachments.add(new AttachmentInstanceImpl(attachmentInstance));
    }
		return attachments;
	}

	public byte[] getAttachmentValue(AttachmentInstance attachmentInstance, String queryList)
	throws RemoteException {
		LargeDataRepository largeDataRepository = EnvTool.getLargeDataRepository();
		String key = Misc.getAttachmentIndexName(attachmentInstance.getName(), attachmentInstance.getVersionDate());
		List<String> categories = Misc.getAttachmentCategories(attachmentInstance.getProcessInstanceUUID());
		return largeDataRepository.getData(byte[].class, categories, key);
	}

	public List<Comment> getCommentFeed(ProcessInstanceUUID instanceUUID, String queryList) throws RemoteException {
    List<Comment> comments = EnvTool.getAllQueriers(queryList).getCommentFeed(instanceUUID);
    return new ArrayList<Comment>(comments);
  }

  public List<Comment> getActivityInstanceCommentFeed(ActivityInstanceUUID activityUUID, String queryList)
  throws RemoteException {
    FacadeUtil.checkArgsNotNull(activityUUID);
    List<Comment> comments = EnvTool.getAllQueriers(queryList).getActivityInstanceCommentFeed(activityUUID);
    return new ArrayList<Comment>(comments);
  }
	
  public int getNumberOfActivityInstanceComments(ActivityInstanceUUID activityUUID, String queryList)
  throws RemoteException {
    return EnvTool.getAllQueriers(queryList).getNumberOfActivityInstanceComments(activityUUID);
  }
  
  public Map<ActivityInstanceUUID, Integer> getNumberOfActivityInstanceComments(Set<ActivityInstanceUUID> activityUUIDs, String queryList)
  throws RemoteException {
    if(activityUUIDs == null || activityUUIDs.isEmpty()){
      return Collections.emptyMap();
    }
    return EnvTool.getAllQueriers(queryList).getNumberOfActivityInstanceComments(activityUUIDs);
  }

  public int getNumberOfComments(ProcessInstanceUUID instanceUUID, String queryList)
  throws InstanceNotFoundException, RemoteException {
    return EnvTool.getAllQueriers(queryList).getNumberOfComments(instanceUUID);
  }

  public int getNumberOfProcessInstanceComments(ProcessInstanceUUID instanceUUID, String queryList)
  throws RemoteException {
    return EnvTool.getAllQueriers(queryList).getNumberOfProcessInstanceComments(instanceUUID);
  }

  public List<Comment> getProcessInstanceCommentFeed(ProcessInstanceUUID instanceUUID, String queryList)
  throws RemoteException {
    FacadeUtil.checkArgsNotNull(instanceUUID);
    List<Comment> comments = EnvTool.getAllQueriers(queryList).getProcessInstanceCommentFeed(instanceUUID);
    return new ArrayList<Comment>(comments);
  }

  public LightTaskInstance getLightTaskInstance(ActivityInstanceUUID taskUUID, String queryList)
  throws TaskNotFoundException, RemoteException {
    final TaskInstance taskInstance = EnvTool.getAllQueriers(queryList).getTaskInstance(taskUUID);
    if (taskInstance == null) {
      throw new TaskNotFoundException("bai_QRAPII_5", taskUUID);
    }
    return new LightActivityInstanceImpl(taskInstance);
  }

  public LightActivityInstance getLightActivityInstance(ActivityInstanceUUID activityInstanceUUID, String queryList)
      throws ActivityNotFoundException, RemoteException {
    final ActivityInstance activityInstance = EnvTool.getAllQueriers(queryList).getActivityInstance(activityInstanceUUID);
    if (activityInstance == null) {
      throw new ActivityNotFoundException("bai_QRAPII_11", activityInstanceUUID);
    }
    return new LightActivityInstanceImpl(activityInstance);
  }

}
