/**
 * Copyright (C) 2009  BonitaSoft S.A.
 * BonitaSoft, 31 rue Gustave Eiffel - 38000 Grenoble
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade.impl;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;

import org.ow2.bonita.facade.internal.InternalWebAPI;
import org.ow2.bonita.facade.runtime.Label;
import org.ow2.bonita.facade.runtime.WebTemporaryToken;
import org.ow2.bonita.facade.runtime.impl.CaseImpl;
import org.ow2.bonita.facade.runtime.impl.InternalProcessInstance;
import org.ow2.bonita.facade.runtime.impl.LabelImpl;
import org.ow2.bonita.facade.runtime.impl.WebTemporaryTokenImpl;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.light.LightProcessInstance;
import org.ow2.bonita.light.impl.LightProcessInstanceImpl;
import org.ow2.bonita.services.WebService;
import org.ow2.bonita.services.WebTokenManagementService;
import org.ow2.bonita.util.BonitaConstants;
import org.ow2.bonita.util.EnvTool;
import org.ow2.bonita.util.Misc;

/**
 * @author Charles Souillard, Matthieu Chaffotte, Nicolas Chabanoles
 */
public class WebAPIImpl implements InternalWebAPI {

  protected WebAPIImpl() { }

  public void deletePhantomCases() throws RemoteException {
    Set<ProcessInstanceUUID> webCases = EnvTool.getWebService().getAllCases();
    Set<ProcessInstanceUUID> runtimeCases = EnvTool.getAllQueriers().getParentInstancesUUIDs();

    webCases.removeAll(runtimeCases);
    if (!webCases.isEmpty()) {
      EnvTool.getWebService().deleteCases(webCases);
    }
  }

  public void addLabel(String labelName, String ownerName,
      String editableCSSStyleName, String readonlyCSSStyleName,
      String previewCSSStyleName, boolean isVisible, boolean hasToBeDisplayed,
      String iconCSSStyle, Set<ProcessInstanceUUID> caseList, int displayOrder,
      boolean isSystemLabel) throws RemoteException {
    final WebService webService = EnvTool.getWebService();
    final LabelImpl label = new LabelImpl(labelName, ownerName, editableCSSStyleName, readonlyCSSStyleName, previewCSSStyleName, isVisible, hasToBeDisplayed, iconCSSStyle, displayOrder, isSystemLabel);
    webService.addLabel(label);

    addCasesToLabel(ownerName, labelName, caseList);
  }

  public void removeLabel(String ownerName, String labelName) throws RemoteException {
    EnvTool.getWebService().removeLabel(ownerName, labelName);
  }

  public void removeCasesFromLabel(String ownerName, String labelName, Set<ProcessInstanceUUID> caseList) {
    final Set<InternalProcessInstance> instances = EnvTool.getAllQueriers().getProcessInstances(caseList);
    final WebService webService = EnvTool.getWebService();
    for (InternalProcessInstance parentNewOrUpdatedInstance : instances) {
      webService.removeCase(parentNewOrUpdatedInstance.getUUID(), ownerName, labelName);

    }

    EnvTool.getWebService().removeCasesFromLabel(ownerName, labelName, caseList);
  }

  public void deleteAllCases() throws RemoteException {
    EnvTool.getWebService().deleteAllCases();
  }

  public void updateLabelCSS(String ownerName, String labelName,
      String aEditableCSSStyle, String aPreviewCSSStyle,
      String aReadOnlyCSSStyle) throws RemoteException { 
    LabelImpl label = EnvTool.getWebService().getLabel(ownerName, labelName);
    label.setEditableCSSStyleName(aEditableCSSStyle);
    label.setPreviewCSSStyleName(aPreviewCSSStyle);
    label.setReadonlyCSSStyleName(aReadOnlyCSSStyle);
  }

  public void addCasesToLabel(String ownerName, String labelName, Set<ProcessInstanceUUID> caseList) {
    final Set<InternalProcessInstance> instances = EnvTool.getAllQueriers().getProcessInstances(caseList);
    final WebService webService = EnvTool.getWebService();
    for (InternalProcessInstance internalProcessInstance : instances) {
      webService.addCase(new CaseImpl(internalProcessInstance.getUUID(), ownerName, labelName));
    }
  }

  public Label getLabel(String ownerName, String labelName) throws RemoteException {
    LabelImpl label = EnvTool.getWebService().getLabel(ownerName, labelName);
    if (label == null) {
      return null;
    }
    return new LabelImpl(label);
  }

  public Set<Label> getLabels(String ownerName) throws RemoteException {
    Set<LabelImpl> labels = EnvTool.getWebService().getLabels(ownerName);
    return getCopy(labels);
  }

  public List<Label> getSystemLabels(String ownerName) throws RemoteException {
    List<LabelImpl> labels = EnvTool.getWebService().getSystemLabels(ownerName);
    return getCopy(labels);
  }

  public List<Label> getUserCustomLabels(String ownerName) throws RemoteException {
    List<LabelImpl> labels = EnvTool.getWebService().getUserCustomLabels(ownerName);
    return getCopy(labels);
  }

  public Set<Label> getCaseLabels(String ownerName, ProcessInstanceUUID case_) {
    Set<LabelImpl> labels = EnvTool.getWebService().getCaseLabels(ownerName, case_);
    return getCopy(labels);
  }

  public Map<ProcessInstanceUUID, Set<Label>> getCasesLabels(String ownerName, Set<ProcessInstanceUUID> cases) {
    if (cases == null || cases.isEmpty()) {
      return Collections.emptyMap();
    }
    Map<ProcessInstanceUUID, Set<Label>> result = new HashMap<ProcessInstanceUUID, Set<Label>>();
    for (ProcessInstanceUUID case_ : cases) {
      result.put(case_, getCaseLabels(ownerName, case_));
    }
    return result;
  }

  public void removeAllCasesFromLabels(Set<ProcessInstanceUUID> caseList) {
    if (caseList != null && !caseList.isEmpty()) {
      final WebService webService = EnvTool.getWebService();
      for (ProcessInstanceUUID case_ : caseList) {
        webService.removeCase(case_);
      }
    }
  }

  public Set<ProcessInstanceUUID> getCases(String ownerName, Set<String> labels) throws RemoteException {
    if (ownerName != null && labels != null && !labels.isEmpty()) {
      final WebService webService = EnvTool.getWebService();
      Set<ProcessInstanceUUID> cases = webService.getCases(ownerName, labels);
      if (cases == null) {
        return Collections.emptySet();    
      }
      Set<ProcessInstanceUUID> result = new HashSet<ProcessInstanceUUID>();
      for (ProcessInstanceUUID instanceUUID : cases) {
        result.add(new ProcessInstanceUUID(instanceUUID));
      }
      return result;
    }
    return Collections.emptySet();
  }

  public List<LightProcessInstance> getLightProcessInstances(final String ownerName, final Set<String> theLabelsName, final int fromIndex, final int pageSize)
  throws RemoteException {
    if (ownerName == null || theLabelsName == null || theLabelsName.isEmpty()) {
      return Collections.emptyList();
    }

    List<ProcessInstanceUUID> caseUUIDs = EnvTool.getWebService().getLabelsCaseUUIDs(ownerName, theLabelsName, fromIndex, pageSize);
    Set<InternalProcessInstance> internalList = EnvTool.getAllQueriers().getProcessInstances(caseUUIDs);

    if (internalList == null || internalList.isEmpty()) {
      return Collections.emptyList();
    }
    Comparator<InternalProcessInstance> comparator = new Comparator<InternalProcessInstance>() {
      public int compare(InternalProcessInstance o1, InternalProcessInstance o2) {
        return o2.getLastUpdate().compareTo(o1.getLastUpdate());
      }
    };
    List<InternalProcessInstance> sortedInstances = new ArrayList<InternalProcessInstance>(internalList);
    Collections.sort(sortedInstances, comparator);

    List<LightProcessInstance> list = new ArrayList<LightProcessInstance>(); 
    for (InternalProcessInstance internalProcessInstance : sortedInstances) {
      list.add(new LightProcessInstanceImpl(internalProcessInstance));
    }

    return list;
  }

  public boolean setLabelsOnNewOrUpdatedCases(String ownerName, Set<String> labelsToApplyOnNewOrUpdated, Set<String> labelsToRemoveOnNewOrUpdated, String labelToApplyOnMyCases) {
    boolean continue_ = false;
    final WebService webService = EnvTool.getWebService();

    final Set<InternalProcessInstance> newOrUpdatedProcessInstances = EnvTool.getAllQueriers().getUpdatedWebProcessInstances(ownerName, 0, BonitaConstants.MAX_LIST_SIZE);
    final Set<ProcessInstanceUUID> caseUUIDs = new HashSet<ProcessInstanceUUID>();
    if (newOrUpdatedProcessInstances != null && !newOrUpdatedProcessInstances.isEmpty()) {
      if (newOrUpdatedProcessInstances.size() >= BonitaConstants.MAX_LIST_SIZE) {
        continue_ = true;
      }

      final Set<CaseImpl> casesToAdd = new HashSet<CaseImpl>();

      final Map<ProcessInstanceUUID, Date> rootInstancesUpdates = new HashMap<ProcessInstanceUUID, Date>();
      final Set<InternalProcessInstance> rootInstances = new HashSet<InternalProcessInstance>();

      for (InternalProcessInstance instance : newOrUpdatedProcessInstances) {
        final ProcessInstanceUUID rootInstanceUUID = instance.getRootInstanceUUID();
        InternalProcessInstance rootInstance = null;
        if (rootInstanceUUID.equals(instance.getUUID())) {
          rootInstance = instance;
        } else {
          rootInstance = EnvTool.getAllQueriers().getProcessInstance(rootInstanceUUID);
        }
        caseUUIDs.add(rootInstanceUUID);        
        if (labelsToApplyOnNewOrUpdated != null) {
          for (String label : labelsToApplyOnNewOrUpdated) {
            final CaseImpl case_ = new CaseImpl(rootInstance.getUUID(), ownerName, label);
            casesToAdd.add(case_);
          }
        }
        rootInstancesUpdates.put(rootInstanceUUID, instance.getLastUpdate());
        rootInstances.add(rootInstance);
      }
      //ends with read operations

      final Set<CaseImpl> cases = webService.getCases(caseUUIDs);

      if (labelToApplyOnMyCases != null) {
        Set<ProcessInstanceUUID> myCases = Collections.emptySet();

        final Set<CaseImpl> myCasesImpl = webService.getLabelCases(ownerName, getSetFromString(labelToApplyOnMyCases), caseUUIDs);
        if (myCasesImpl != null) {
          myCases = new HashSet<ProcessInstanceUUID>();
          for (CaseImpl caseImpl : myCasesImpl) {
            myCases.add(caseImpl.getUUID());
          }
        }
        for (InternalProcessInstance rootInstance : rootInstances) {
          if (!myCases.contains(rootInstance.getUUID())) {
            CaseImpl case_ = new CaseImpl(rootInstance.getUUID(), rootInstance.getStartedBy(), labelToApplyOnMyCases);
            casesToAdd.add(case_);
          }
        }
      }
      if (labelsToApplyOnNewOrUpdated != null) {
        casesToAdd.removeAll(webService.getLabelCases(ownerName, labelsToApplyOnNewOrUpdated, caseUUIDs));
      }
      Set<CaseImpl> casesToRemove = Collections.emptySet(); 
      if (labelsToRemoveOnNewOrUpdated != null && !labelsToRemoveOnNewOrUpdated.isEmpty()) {
        casesToRemove = webService.getLabelCases(ownerName, labelsToRemoveOnNewOrUpdated, caseUUIDs);
      }

      //end of read operations, start write operations
      for (CaseImpl caseImpl : casesToRemove) {
        webService.removeCase(caseImpl);  
      }
      for (CaseImpl caseImpl : casesToAdd) {
        caseImpl.setLastUpdate(rootInstancesUpdates.get(caseImpl.getUUID()).getTime());
        webService.addCase(caseImpl);
      }
      for (InternalProcessInstance instance : newOrUpdatedProcessInstances) {
        instance.setUserXPRead(instance.getLastUpdate());
      }
      cases.removeAll(casesToRemove);
      for (CaseImpl caseImpl : cases) {
        caseImpl.setLastUpdate(rootInstancesUpdates.get(caseImpl.getUUID()).getTime());
      }
    }
    return continue_;
  }

  private Set<String> getSetFromString(final String s) {
    final Set<String> result = new HashSet<String>();
    result.add(s);
    return result;
  }
  public Set<Label> getLabels(String ownerName, Set<String> labelsName) throws RemoteException {
    Set<LabelImpl> labels = EnvTool.getWebService().getLabels(ownerName, labelsName);
    return getCopy(labels);
  }

  private Set<Label> getCopy(Set<LabelImpl> labels) {
    if (labels == null || labels.isEmpty()) {
      return Collections.emptySet();
    }
    Set<Label> result = new HashSet<Label>();
    for (Label label : labels) {
      result.add(new LabelImpl(label));
    }
    return result;
  }

  private List<Label> getCopy(List<LabelImpl> labels) {
    if (labels == null || labels.isEmpty()) {
      return Collections.emptyList();
    }
    List<Label> result = new ArrayList<Label>();
    for (Label label : labels) {
      result.add(new LabelImpl(label));
    }
    return result;
  }

  public void removeLabels(String ownerName, Collection<String> labelNames) throws RemoteException {
    for (String labelName : labelNames) {
      removeLabel(ownerName, labelName);
    }
  }

  public void updateLabelName(String ownerName, String labelName, String newName) throws RemoteException {
    EnvTool.getWebService().updateLabelName(ownerName, labelName, newName);
  }

  public void updateLabelVisibility(String ownerName, String labelName, boolean isVisible) { 
    LabelImpl label = EnvTool.getWebService().getLabel(ownerName, labelName);
    label.setVisible(isVisible);
  }

  public void updateLabelVisibility(String ownerName, Map<String, Boolean> labelvisibilities)
  throws RemoteException {
    for (Entry<String, Boolean> labelVisibility : labelvisibilities.entrySet()) {
      updateLabelVisibility(ownerName, labelVisibility.getKey(), labelVisibility.getValue());
    }
  }

  public Map<String, Integer> getCasesNumber(String ownerName, Collection<String> labelNames, int limit) throws RemoteException {
    final Map<String, Integer> result = new HashMap<String, Integer>();
    final WebService webService = EnvTool.getWebService();
    for (String label : labelNames) {
      int number = 0;
      if (limit <= BonitaConstants.MAX_LIST_SIZE) {
        final Set<CaseImpl> cases = webService.getCases(ownerName, label, limit + 1);
        number = cases.size();
        if (number > limit) {
          number = -limit;
        }
      } else {
        number = webService.getCasesNumber(ownerName, label);
      }
      result.put(label, number);
    }
    return result;
  }
  
  public Map<String, Integer> getCasesNumber(String ownerName, Collection<String> labelNames) throws RemoteException {
    final Map<String, Integer> result = new HashMap<String, Integer>();
    final WebService webService = EnvTool.getWebService();
    for (String label : labelNames) {
      int number = webService.getCasesNumber(ownerName, label);
      result.put(label, number);
    }
    return result;
  }

  public Map<String, Integer> getCasesNumber(String ownerName, String labelName, Collection<String> labelNames, int limit) throws RemoteException {
    final Map<String, Integer> result = new HashMap<String, Integer>();
    final WebService webService = EnvTool.getWebService();
    for (String label : labelNames) {
      int number = 0;
      if (limit <= BonitaConstants.MAX_LIST_SIZE) {
        final Set<CaseImpl> cases = webService.getCases(ownerName, labelName, label, limit + 1);
        number = cases.size();
        if (number > limit) {
          number = -limit;
        }
      } else {
        number = webService.getCasesNumber(ownerName, labelName, label);
      }
      result.put(label, number);
    }
    return result;
  }

  public String generateTemporaryToken(String identityKey) throws RemoteException {
    FacadeUtil.checkArgsNotNull(identityKey);
    if(identityKey.length()==0){
      throw new IllegalArgumentException();
    }
    WebTokenManagementService webTokenManagementService = EnvTool.getWebTokenManagementService();
    String tokenKey = Misc.getUniqueId("");
    // Token will expire in 1 hour.
    long expirationDate = new Date().getTime() + (1000 * 60 * 60);
    WebTemporaryTokenImpl temporaryToken = new WebTemporaryTokenImpl(tokenKey,expirationDate,identityKey);
    webTokenManagementService.addTemporaryToken(temporaryToken);
    return temporaryToken.getToken();
  }

  public String getIdentityKeyFromTemporaryToken(String token) throws RemoteException {
    WebTokenManagementService webTokenManagementService = EnvTool.getWebTokenManagementService();
    WebTemporaryToken temporaryToken = webTokenManagementService.getToken(token);
    if(temporaryToken==null){
      return null;
    }
    // Token can be used only once.
    webTokenManagementService.deleteToken((WebTemporaryTokenImpl)temporaryToken);

    // Clean all expired tokens
    Set<WebTemporaryToken> expiredTokens = webTokenManagementService.getExpiredTokens();
    for (WebTemporaryToken expiredToken : expiredTokens) {
      webTokenManagementService.deleteToken((WebTemporaryTokenImpl)expiredToken);
    }

    return temporaryToken.getIdentityKey();
  }

}
