/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade.interceptor;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import javax.interceptor.InvocationContext;

import org.ow2.bonita.facade.impl.StandardInternalAPIAccessorImpl;
import org.ow2.bonita.facade.internal.InternalAPIAccessor;
import org.ow2.bonita.util.BonitaRuntimeException;
import org.ow2.bonita.util.ExceptionManager;

/**
 * Interceptor class to redirect ejb3 bean calls to the standalone api
 * @author Thomas Gueze
 *
 */
public class EJB3Interceptor {

  private static final int SUFFIX_LENGTH = "Bean".length();

  private Object getStandaloneAPI(final String className) {
    try {
      final InternalAPIAccessor apiAccessor = new StandardInternalAPIAccessorImpl();
      final Method m = apiAccessor.getClass().getMethod("get" + className);
      return m.invoke(apiAccessor);
    } catch (final Exception e) {
    	String message = ExceptionManager.getInstance().getFullMessage("baie_EJB3I_1");
      throw new BonitaRuntimeException(message, e);
    }
  }

  public Object performInterception(final InvocationContext ctx) throws Exception {
    String className = ctx.getTarget().getClass().getSimpleName();
    className = className.substring(0, className.length() - SUFFIX_LENGTH);
    final Object standaloneAPI = getStandaloneAPI(className);
    Method implMethod = null;
    try {
      implMethod = standaloneAPI.getClass().getMethod(ctx.getMethod().getName(), ctx.getMethod().getParameterTypes());
      return implMethod.invoke(standaloneAPI, ctx.getParameters());
    } catch (final Exception e) {
      if (e instanceof InvocationTargetException && implMethod != null) {
        final Throwable invocationExceptionCause = e.getCause();
        InterceptorUtil.manageInvokeExceptionCause(implMethod, invocationExceptionCause);
      }
      String message = ExceptionManager.getInstance().getFullMessage("baie_EJB3I_2", ctx.getMethod().getName());
      throw new BonitaRuntimeException(message, e);
    }
  }
}
