/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 * 
 * Modified by Matthieu Chaffotte - BonitaSoft S.A.
 **/
package org.ow2.bonita.facade.internal;

import java.rmi.Remote;
import java.rmi.RemoteException;
import java.util.List;
import java.util.Set;

import org.ow2.bonita.facade.def.element.AttachmentDefinition;
import org.ow2.bonita.facade.def.element.BusinessArchive;
import org.ow2.bonita.facade.def.majorElement.ActivityDefinition;
import org.ow2.bonita.facade.def.majorElement.DataFieldDefinition;
import org.ow2.bonita.facade.def.majorElement.ParticipantDefinition;
import org.ow2.bonita.facade.def.majorElement.ProcessDefinition;
import org.ow2.bonita.facade.exception.ActivityDefNotFoundException;
import org.ow2.bonita.facade.exception.ActivityNotFoundException;
import org.ow2.bonita.facade.exception.DataFieldNotFoundException;
import org.ow2.bonita.facade.exception.ParticipantNotFoundException;
import org.ow2.bonita.facade.exception.ProcessNotFoundException;
import org.ow2.bonita.facade.runtime.InitialAttachment;
import org.ow2.bonita.facade.uuid.ActivityDefinitionUUID;
import org.ow2.bonita.facade.uuid.ParticipantDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.light.LightProcessDefinition;

/**
 * For internal use only.
 */
public interface InternalQueryDefinitionAPI extends Remote {

  ProcessDefinition getProcess(String processId, String processVersion, String queryList)
    throws ProcessNotFoundException, RemoteException;

  Set<ProcessDefinition> getProcesses(String queryList) throws RemoteException;
  Set<LightProcessDefinition> getLightProcesses(String queryList) throws RemoteException;

  int getNumberOfProcesses(String queryList) throws RemoteException;
  Set<ProcessDefinition> getProcesses(String processId, String queryList) throws RemoteException;

  List<LightProcessDefinition> getLightProcesses(int fromIndex, int pageSize, String queryList) throws RemoteException;
  
  List<ProcessDefinition> getProcesses(int fromIndex, int pageSize, String queryList) throws RemoteException;
  
  ProcessDefinition getProcess(ProcessDefinitionUUID processDefinitionUUID, String queryList) throws RemoteException, ProcessNotFoundException;
  LightProcessDefinition getLightProcess(ProcessDefinitionUUID processDefinitionUUID, String queryList) throws RemoteException, ProcessNotFoundException;
  
  Set<ProcessDefinition> getProcesses(ProcessDefinition.ProcessState processState, String queryList) throws RemoteException;
  Set<LightProcessDefinition> getLightProcesses(ProcessDefinition.ProcessState processState, String queryList) throws RemoteException;
  
  Set<ProcessDefinition> getProcesses(String processId, ProcessDefinition.ProcessState processState, String queryList) 
    throws RemoteException;
  
  BusinessArchive getBusinessArchive(ProcessDefinitionUUID processDefinitionUUID, String queryList) throws ProcessNotFoundException, RemoteException;
  
  Set<ParticipantDefinition> getProcessParticipants(ProcessDefinitionUUID processDefinitionUUID, String queryList) 
    throws RemoteException, ProcessNotFoundException;
  
  ParticipantDefinition getProcessParticipant(ProcessDefinitionUUID processDefinitionUUID,
      String participantId, String queryList) throws RemoteException, ProcessNotFoundException, ParticipantNotFoundException;

  Set<ActivityDefinition> getProcessActivities(ProcessDefinitionUUID processDefinitionUUID, String queryList) 
    throws RemoteException, ProcessNotFoundException;
  
  ActivityDefinition getProcessActivity(ProcessDefinitionUUID processDefinitionUUID, String activityId, String queryList) 
    throws RemoteException, ProcessNotFoundException, ActivityNotFoundException;
  
  ActivityDefinitionUUID getProcessActivityId(ProcessDefinitionUUID processDefinitionUUID,
      String activityName, String queryList) throws RemoteException, ProcessNotFoundException;

  ParticipantDefinitionUUID getProcessParticipantId(ProcessDefinitionUUID processDefinitionUUID,
      String participantName, String queryList) throws RemoteException, ProcessNotFoundException;
    
  ProcessDefinition getLastProcess(String processId, 
      String queryList) throws RemoteException, ProcessNotFoundException;

  Set<DataFieldDefinition> getProcessDataFields(ProcessDefinitionUUID processDefinitionUUID, String queryList)
    throws RemoteException, ProcessNotFoundException;

  DataFieldDefinition getProcessDataField(ProcessDefinitionUUID processDefinitionUUID, String dataFieldId, String queryList)
    throws RemoteException, ProcessNotFoundException, DataFieldNotFoundException;

  Set<DataFieldDefinition> getActivityDataFields(ActivityDefinitionUUID uuid, String queryList)
    throws RemoteException, ActivityDefNotFoundException;
  
  DataFieldDefinition getActivityDataField(ActivityDefinitionUUID activityDefinitionUUID, String dataFieldId, String queryList)
    throws RemoteException, ActivityDefNotFoundException, DataFieldNotFoundException;
  
  String getProcessMetaData(ProcessDefinitionUUID uuid, String key, String queryList) throws RemoteException, ProcessNotFoundException;
  
  Set<InitialAttachment> getProcessAttachments(ProcessDefinitionUUID processUUID, String queryList) throws ProcessNotFoundException, RemoteException;
  InitialAttachment getProcessAttachment(ProcessDefinitionUUID processUUID, String attachmentName, String queryList) throws ProcessNotFoundException, RemoteException;
  
  Set<AttachmentDefinition> getAttachmentDefinitions(ProcessDefinitionUUID processUUID, String queryList) throws ProcessNotFoundException, RemoteException;
  AttachmentDefinition getAttachmentDefinition(ProcessDefinitionUUID processUUID, String attachmentName, String queryList) throws ProcessNotFoundException, RemoteException;

  byte[] getResource(ProcessDefinitionUUID definitionUUID, String resourcePath, String queryList) throws ProcessNotFoundException, RemoteException;

  Set<LightProcessDefinition> getLightProcesses(Set<ProcessDefinitionUUID> processUUIDs, String queryList) throws ProcessNotFoundException, RemoteException;

  List<LightProcessDefinition> getAllLightProcessesExcept(Set<ProcessDefinitionUUID> processUUIDs, int fromIndex, int pageSize, String queryList) throws RemoteException;
}
