/**
 * Copyright (C) 2010  BonitaSoft S.A.
 * BonitaSoft, 31 rue Gustave Eiffel - 38000 Grenoble
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade.privilege.impl;

import java.util.HashSet;
import java.util.Set;

import org.ow2.bonita.facade.def.majorElement.impl.DescriptionElementImpl;
import org.ow2.bonita.facade.privilege.Rule;
import org.ow2.bonita.facade.uuid.AbstractUUID;
import org.ow2.bonita.util.Misc;

/**
 * @author Nicolas Chabanoles
 * 
 */
public abstract class RuleImpl extends DescriptionElementImpl implements Rule {

  /**
   * UID
   */
  private static final long serialVersionUID = -5403223858809095337L;

  protected long dbid;

  protected String name;

  protected String label;

  protected Set<String> exceptions;

  protected String type;

  protected Set<String> entities;

  protected RuleImpl(RuleType type) {
    this.type = type.name();
  };

  protected RuleImpl(String name, String label, String description,
      Set<? extends AbstractUUID> items, RuleType type) {
    Misc.checkArgsNotNull(name, items, type);
    this.name = name;
    this.label = label;
    if(items != null){
      this.exceptions = new HashSet<String>();
      for (AbstractUUID abstractUUID : items) {
        this.exceptions.add(abstractUUID.getValue());
      }
      
    } else {
      this.exceptions = null;
    }

    this.type = type.name();
    setDescription(description);
  }

  protected RuleImpl(Rule src) {
    super(src);
    this.dbid = src.getId();
    this.name = src.getName();
    this.label = src.getLabel();
    this.exceptions = new HashSet<String>();
    Set<String> items_ = src.getItems();
    for (String item : items_) {
      this.exceptions.add(item);
    }
    this.type = src.getType().name();
    this.entities = new HashSet<String>();
    Set<String> entities_ = src.getEntities();
    for (String entity : entities_) {
      this.entities.add(entity);
    }
  }

  public long getId() {
    return dbid;
  }

  public String getName() {
    return this.name;
  }

  public void setName(final String name) {
    this.name = name;
  }

  public String getLabel() {
    return this.label;
  }

  public void setLabel(final String label) {
    this.label = label;
  }

  /**
   * @return a non null set.
   */
  public Set<String> getItems() {
    if (this.exceptions == null) {
      this.exceptions = new HashSet<String>();
    }
    return this.exceptions;
  }

  public RuleType getType() {
    return RuleType.valueOf(this.type);
  }

  /**
   * 
   * @param items
   *          must be not null
   */
  public void addItems(Set<String> items) {
    Misc.checkArgsNotNull(items);
    if (this.exceptions == null) {
      this.exceptions = new HashSet<String>();
    }
    this.exceptions.addAll(items);

  }

  /**
   * 
   * @param exceptions
   *          must be not null
   */
  public void removeExceptions(Set<String> exceptions) {
    Misc.checkArgsNotNull(exceptions);
    if (this.exceptions!=null && !this.exceptions.isEmpty()) {
      this.exceptions.removeAll(exceptions);
    }
  }

  public void setExceptions(Set<String> exceptions) {
    Misc.checkArgsNotNull(exceptions);
    this.exceptions = exceptions;
  }

  public Set<String> getEntities() {
    if (this.entities == null) {
      this.entities = new HashSet<String>();
    }
    return this.entities;
  }
  
  /**
   * 
   * @param entities
   *          must be not null
   */
  public void addEntitiess(Set<String> entities) {
    Misc.checkArgsNotNull(entities);
    if (this.entities == null) {
      this.entities = new HashSet<String>();
    }
    this.entities.addAll(entities);

  }

  /**
   * 
   * @param entities
   *          must be not null
   */
  public void removeEntities(Set<String> entities) {
    Misc.checkArgsNotNull(entities);
    if (this.entities!=null && !this.entities.isEmpty()) {
      this.entities.removeAll(entities);
    }
  }
  
  @Override
  public String toString() {
    return this.name + " - " + this.label + " - " + this.type;
  }
}
