/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 * 
 * Modified by Matthieu Chaffotte - BonitaSoft S.A.
 **/
package org.ow2.bonita.facade.runtime.impl;

import java.util.Set;

import org.ow2.bonita.facade.def.majorElement.ActivityDefinition;
import org.ow2.bonita.facade.runtime.ActivityInstance;
import org.ow2.bonita.facade.runtime.ActivityState;
import org.ow2.bonita.facade.uuid.ActivityInstanceUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.services.Querier;
import org.ow2.bonita.util.EnvTool;

/**
 * @author Pierre Vigneras
 */
public class InternalActivityInstance extends ActivityInstanceImpl {

  private static final long serialVersionUID = -8515098234372896097L;
  protected long dbid;

  protected InternalActivityInstance() { }

  public InternalActivityInstance(final ActivityInstanceUUID uuid, final ActivityDefinition activityDefinition,
      final ProcessDefinitionUUID processUUID, final ProcessInstanceUUID instanceUUID, final ProcessInstanceUUID rootInstanceUUID,
      final String iterationId, final String activityInstanceId) {
    super(uuid, activityDefinition, processUUID, instanceUUID, rootInstanceUUID, iterationId, activityInstanceId);
  }

  public InternalActivityInstance(final ActivityInstance src) {
    super(src);
  }

  protected void updateLastUpdateDate() {
    super.updateLastUpdateDate();
    final Querier querier = EnvTool.getJournalQueriers();
    querier.getProcessInstance(getProcessInstanceUUID()).updateLastUpdateDate();
  }


  @Override
  public void setTaskAssign(ActivityState taskState, String loggedInUserId, String assignedUserId) {
    super.setTaskAssign(taskState, loggedInUserId, assignedUserId);
    if (assignedUserId != null) {
      final Querier querier = EnvTool.getJournalQueriers();
      final InternalProcessInstance instance = querier.getProcessInstance(getProcessInstanceUUID()); 
      if (assignedUserId != null) {
        instance.addInvolvedUser(assignedUserId);
      }
    }
  }
  
  @Override
  public void setTaskAssign(ActivityState taskState, String loggedInUserId, Set<String> candidates) {
    super.setTaskAssign(taskState, loggedInUserId, candidates);
    if (candidates != null && !candidates.isEmpty()) {
      final Querier querier = EnvTool.getJournalQueriers();
      final InternalProcessInstance instance = querier.getProcessInstance(getProcessInstanceUUID()); 
      if (candidates != null) {
        instance.addInvolvedUsers(candidates);
      }
    }
  }
  
  public void setSubflowProcessInstanceUUID(final ProcessInstanceUUID subflowProcessInstanceUUID) {
    this.subflowProcessInstanceUUID = subflowProcessInstanceUUID;
  }
  
  public void setPriority(int priority) {
    this.priority = priority;
  }
  
  public void setDynamicDescription(String dynamicDescription) {
    this.dynamicDescription = dynamicDescription;
  }

  public void setDynamicLabel(String dynamicLabel) {
    this.dynamicLabel = dynamicLabel;
  }
}
