package org.ow2.bonita.runtime.event;

import java.util.logging.Level;
import java.util.logging.Logger;

import org.ow2.bonita.env.Authentication;
import org.ow2.bonita.env.Environment;
import org.ow2.bonita.env.Transaction;
import org.ow2.bonita.facade.impl.StandardAPIAccessorImpl;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.services.EventService;
import org.ow2.bonita.util.BonitaConstants;
import org.ow2.bonita.util.BonitaRuntimeException;
import org.ow2.bonita.util.Command;
import org.ow2.bonita.util.EnvTool;

public class ExecuteEventsCouplesCommand implements Command<Void> {
  
  private static final Logger LOG = Logger.getLogger(ExecuteEventsCouplesCommand.class.getName());
  
  private static final long serialVersionUID = -479276850307735480L;

  private EventCoupleId eventCouple;
  private EventExecutor eventExecutor;

  public ExecuteEventsCouplesCommand(EventCoupleId eventCouple, EventExecutor eventExecutor) {
    this.eventCouple = eventCouple;
    this.eventExecutor = eventExecutor;
  }

  public Void execute(Environment environment) throws Exception {
    OutgoingEventInstance outgoing = null;
    IncomingEventInstance incoming = null;
    try {
      final EventService eventService = EnvTool.getEventService();

      outgoing = eventService.getOutgoingEvent(eventCouple.getOutgoing());
      incoming = eventService.getIncomingEvent(eventCouple.getIncoming());

      if (incoming != null && outgoing != null) {
        final String executionEventUUID = incoming.getExecutionUUID();
        if (executionEventUUID == null) {
          //it is a process start.
          final ProcessDefinitionUUID processUUID = incoming.getActivityDefinitionUUID().getProcessUUID();
          Authentication.setUserId(BonitaConstants.SYSTEM_USER);
          final ProcessInstanceUUID instanceUUID = new StandardAPIAccessorImpl().getRuntimeAPI().instantiateProcess(processUUID);
          outgoing.setLockOwner(instanceUUID.toString());
          incoming.setLockOwner(null);
        } else {
          this.eventExecutor.getCommandService().execute(new ExecuteIncomingEventInstanceCommand(eventCouple));
        }
      }
    } catch (Throwable exception) {
      if (LOG.isLoggable(Level.INFO)) {
        LOG.info("Exception caught while executing eventCouple: " + exception + ". Exception: " + exception.getMessage());
      }
      if (LOG.isLoggable(Level.INFO)) {
        LOG.info("handling events, incoming: " + incoming + ", outgoing: " + outgoing + " exception: " + exception.getMessage());
      }

      handleException(environment, eventCouple, exception);
    }
    return null;
  }
  
  protected void handleException(Environment environment, EventCoupleId eventCouple, Throwable exception) {
    Transaction transaction = environment.get(Transaction.class);

    EventCoupleExceptionHandler handler = new EventCoupleExceptionHandler(eventCouple, exception, eventExecutor.getCommandService());
    transaction.registerSynchronization(handler);
    
    if (exception instanceof RuntimeException) {
      throw (RuntimeException) exception;
    }
    throw new BonitaRuntimeException("Execution of event couple (" + eventCouple + ") failed: " + exception.getMessage(), exception);
  }
}
