package org.ow2.bonita.runtime.event;

import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.ow2.bonita.env.Environment;
import org.ow2.bonita.runtime.model.Execution;
import org.ow2.bonita.services.EventService;
import org.ow2.bonita.services.Querier;
import org.ow2.bonita.util.BonitaRuntimeException;
import org.ow2.bonita.util.Command;
import org.ow2.bonita.util.EnvTool;

public class ExecuteIncomingEventInstanceCommand implements Command<Void> {

  private static final long serialVersionUID = 1L;
  private static final Logger LOG = Logger.getLogger(ExecuteIncomingEventInstanceCommand.class.getName());
  private EventCoupleId eventCouple;
  
  public ExecuteIncomingEventInstanceCommand(final EventCoupleId eventCouple) {
    this.eventCouple = eventCouple;  
  }
  
  public Void execute(Environment environment) throws Exception {
    final Querier journal = EnvTool.getJournalQueriers();
    final EventService eventService = EnvTool.getEventService();
    OutgoingEventInstance outgoing = eventService.getOutgoingEvent(eventCouple.getOutgoing());
    IncomingEventInstance incoming = eventService.getIncomingEvent(eventCouple.getIncoming());
    
    final Execution execution = journal.getExecutionWithEventUUID(incoming.executionUUID);
    if (execution == null) {
      if (LOG.isLoggable(Level.INFO)) {
        LOG.info("No active execution found for uuid: " + incoming.getExecutionUUID());
      }
      throw new BonitaRuntimeException("No active execution found for uuid: " + incoming.getExecutionUUID());
    } else {
      incoming.setLockOwner(null);
      incoming.setConsumed(true);
      outgoing.setLockOwner(null);
      outgoing.setConsumed(true);

      if (LOG.isLoggable(Level.INFO)) {
        LOG.info("Execution found for uuid: " + incoming.getExecutionUUID());
      }
      Map<String, Object> parameters = outgoing.getParameters();
      if (incoming.isExecutionLocked()) {
        if (!Execution.STATE_ACTIVE.equals(execution.getState())) {
          execution.unlock();
        }
      }
      execution.signal(incoming.getSignal(), parameters);
      return null;
    }

  }
}
