/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 * 
 * Modified by Charles Souillard, Matthieu Chaffotte, Nicolas Chabanoles - BonitaSoft S.A.
 **/
package org.ow2.bonita.services;

import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.ow2.bonita.facade.def.InternalActivityDefinition;
import org.ow2.bonita.facade.def.InternalProcessDefinition;
import org.ow2.bonita.facade.def.majorElement.ProcessDefinition.ProcessState;
import org.ow2.bonita.facade.runtime.ActivityState;
import org.ow2.bonita.facade.runtime.Comment;
import org.ow2.bonita.facade.runtime.InstanceState;
import org.ow2.bonita.facade.runtime.TaskInstance;
import org.ow2.bonita.facade.runtime.impl.InternalActivityInstance;
import org.ow2.bonita.facade.runtime.impl.InternalProcessInstance;
import org.ow2.bonita.facade.uuid.ActivityDefinitionUUID;
import org.ow2.bonita.facade.uuid.ActivityInstanceUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.runtime.model.Execution;

/**
 * @author Pierre Vigneras, Charles Souillard
 */
public interface Querier {

  /*
   * ACTIVITY
   */
  Set<InternalActivityInstance> getActivityInstances(ProcessInstanceUUID instanceUUID);
  InternalActivityInstance getActivityInstance(ProcessInstanceUUID instanceUUID, String activityId, String iterationId, String activityInstanceId);
  InternalActivityInstance getActivityInstance(ActivityInstanceUUID activityInstanceUUID);
  ActivityState getActivityInstanceState(ActivityInstanceUUID activityUUID);
  List<InternalActivityInstance> getActivityInstancesFromRoot(ProcessInstanceUUID rootInstanceUUID);
  List<InternalActivityInstance> getActivityInstancesFromRoot(Set<ProcessInstanceUUID> rootInstanceUUIDs);
  List<InternalActivityInstance> getActivityInstancesFromRoot(Set<ProcessInstanceUUID> rootInstanceUUIDs, ActivityState state);
  Map<ProcessInstanceUUID, InternalActivityInstance> getLastUpdatedActivityInstanceFromRoot(Set<ProcessInstanceUUID> rootInstanceUUIDs, boolean considerSystemTaks);
  Set<InternalActivityInstance> getActivityInstances(ProcessInstanceUUID instanceUUID, String activityName);
  int getNumberOfActivityInstanceComments(ActivityInstanceUUID activityUUID);
  List<Comment> getActivityInstanceCommentFeed(ActivityInstanceUUID activityUUID);
  Map<ActivityInstanceUUID, Integer> getNumberOfActivityInstanceComments(Set<ActivityInstanceUUID> activityUUIDs);

  /*
   * INSTANCE
   */
  Set<InternalProcessInstance> getProcessInstances();
  List<InternalProcessInstance> getMostRecentProcessInstances(int maxResults, long time);
  List<InternalProcessInstance> getMostRecentParentProcessInstances(int maxResults, long time);
  List<InternalProcessInstance> getMostRecentMatchingProcessInstances(Collection<ProcessInstanceUUID> instanceUUIDs, int maxResults, long time);
  List<InternalProcessInstance> getMostRecentProcessesProcessInstances(Collection<ProcessDefinitionUUID> definitionUUIDs, int maxResults, long time);

  List<InternalProcessInstance> getProcessInstances(int fromIndex, int pageSize);
  List<InternalProcessInstance> getProcessInstances(Set<ProcessDefinitionUUID> definitionUUIDs, int fromIndex, int pageSize);
  Set<InternalProcessInstance> getProcessInstances(ProcessDefinitionUUID processUUID);
  Set<InternalProcessInstance> getProcessInstances(Set<ProcessDefinitionUUID> definitionUUIDs);
  Set<InternalProcessInstance> getProcessInstances(ProcessDefinitionUUID processUUID, InstanceState instanceState);
  Set<InternalProcessInstance> getProcessInstances(Collection<ProcessInstanceUUID> instanceUUIDs);
  
  List<InternalProcessInstance> getProcessInstances(Collection<ProcessInstanceUUID> instanceUUIDs, int fromIndex, int pageSize);
  InternalProcessInstance getProcessInstance(ProcessInstanceUUID instanceUUID);
  Set<InternalProcessInstance> getProcessInstancesWithTaskState(Collection<ActivityState> activityStates);
  Set<InternalProcessInstance> getProcessInstancesWithTaskState(Collection<ActivityState> activityStates, Set<ProcessDefinitionUUID> definitionUUIDs);
  Set<InternalProcessInstance> getProcessInstancesWithInstanceStates(Collection<InstanceState> instanceStates, Set<ProcessDefinitionUUID> visibleProcessUUIDs);
  Set<InternalProcessInstance> getProcessInstancesWithInstanceStates(Collection<InstanceState> instanceStates);
  Set<InternalProcessInstance> getUserInstances(String userId);
  Set<InternalProcessInstance> getUserInstances(String userId, Set<ProcessDefinitionUUID> definitionUUIDs);
  List<InternalProcessInstance> getParentProcessInstances(int fromIndex, int pageSize);
  Set<InternalProcessInstance> getUserParentInstances(String userId, Date minStartDate);
  Set<ProcessInstanceUUID> getParentInstancesUUIDs();

  int getNumberOfProcessInstanceComments(ProcessInstanceUUID instanceUUID);
  List<Comment> getProcessInstanceCommentFeed(ProcessInstanceUUID instanceUUID);
  int getNumberOfComments(ProcessInstanceUUID instanceUUID);
  List<Comment> getCommentFeed(ProcessInstanceUUID instanceUUID);

  /**
   * Return all my instances except thos given in parameters
   */
  Set<InternalProcessInstance> getUserInstancesExcept(String userId, Set<ProcessInstanceUUID> myCases);
  long getLastProcessInstanceNb(ProcessDefinitionUUID processUUID);
  Execution getExecutionOnActivity(ProcessInstanceUUID instanceUUID, ActivityInstanceUUID activityInstanceUUID);
  Execution getExecutionWithEventUUID(String eventUUID);
  
  /*
   * TASK
   */
  Set<TaskInstance> getTaskInstances(ProcessInstanceUUID instanceUUID);
  TaskInstance getTaskInstance(ActivityInstanceUUID taskUUID);
  /** return all tasks associated to the given user and having the given state */
  Set<TaskInstance> getUserTasks(String userId, ActivityState taskState);
  Set<TaskInstance> getUserTasks(String userId, ActivityState taskState, Set<ProcessDefinitionUUID> definitionUUIDs);
  Set<TaskInstance> getUserTasks(String userId, Collection<ActivityState> taskStates);
  /** return all tasks associated to the given user and the given instance and having the given state */
  Set<TaskInstance> getUserInstanceTasks(String userId, ProcessInstanceUUID instanceUUID, ActivityState taskState);
  TaskInstance getOneTask(String userId, ProcessInstanceUUID instanceUUID, ActivityState taskState);
  TaskInstance getOneTask(String userId, ProcessDefinitionUUID processUUID, ActivityState taskState);
  TaskInstance getOneTask(String userId, ActivityState taskState);
  TaskInstance getOneTask(String userId, ActivityState taskState, Set<ProcessDefinitionUUID> definitionUUIDs);

  /*
   * DEFINITION
   */
  Set<InternalProcessDefinition> getProcesses();
  Set<InternalProcessDefinition> getProcesses(String processId);
  Set<InternalProcessDefinition> getProcesses(Set<ProcessDefinitionUUID> definitionUUIDs);
  Set<InternalProcessDefinition> getProcesses(Set<ProcessDefinitionUUID> definitionUUIDs, ProcessState processState);
  Set<InternalProcessDefinition> getProcesses(ProcessState processState);
  Set<InternalProcessDefinition> getProcesses(String processId, ProcessState processState);
  InternalProcessDefinition getProcess(String processId, String version);
  InternalProcessDefinition getProcess(ProcessDefinitionUUID processUUID);
  String getLastProcessVersion(String name);
  InternalProcessDefinition getLastDeployedProcess(String processId, ProcessState processState);
  InternalProcessDefinition getLastDeployedProcess(Set<ProcessDefinitionUUID> definitionUUIDs, ProcessState processState);
  Set<InternalProcessDefinition> getDependentProcesses(ProcessDefinitionUUID processUUID);

  InternalActivityDefinition getActivity(ActivityDefinitionUUID activityDefinitionUUID);

  /*
   * WEB
   */
  Set<InternalProcessInstance> getUpdatedWebProcessInstances(String user, int fromIndex, int pageSize);
  Set<InternalProcessInstance> getParentInstances();

  /*
   * BAM
   */
  int getNumberOfOpenSteps();
  int getNumberOfOpenSteps(int priority);
  int getNumberOfOverdueSteps(Date currentDate);
  int getNumberOfStepsAtRisk(Date currentDate, Date atRisk);
  int getNumberOfFinishedSteps(int priority, Date since);

  int getNumberOfUserOpenSteps(String userId);
  int getNumberOfUserOverdueSteps(String userId, Date currentDate);
  int getNumberOfUserStepsAtRisk(String userId, Date currentDate, Date atRisk);
  int getNumberOfUserFinishedSteps(String userId, int priority, Date since);
  int getNumberOfUserOpenSteps(String userId, int priority);

  List<Integer> getNumberOfFinishedCasesPerDay(Date since, Date to);
  List<Integer> getNumberOfExecutingCasesPerDay(Date since, Date to);
  List<Integer> getNumberOfOpenStepsPerDay(Date since, Date to);
  Set<InternalProcessInstance> getUserInstances(String userId, Date minStartDate);
  int getNumberOfParentProcessInstances();
  int getNumberOfParentProcessInstances(Set<ProcessDefinitionUUID> definitionUUIDs);
  int getNumberOfProcessInstances();
  int getNumberOfProcessInstances(Set<ProcessDefinitionUUID> definitionUUIDs);
  int getNumberOfProcesses();
  List<InternalProcessDefinition> getProcesses(int fromIndex, int pageSize);
  List<InternalProcessDefinition> getProcesses(Set<ProcessDefinitionUUID> definitionUUIDs, int fromIndex, int pageSize);
  List<InternalProcessDefinition> getProcessesExcept(Set<ProcessDefinitionUUID> processUUIDs, int fromIndex, int pageSize);
  
  Set<ProcessDefinitionUUID> getAllProcessDefinitionUUIDsExcept(Set<ProcessDefinitionUUID> processUUIDs);
  Set<ProcessDefinitionUUID> getAllProcessDefinitionUUIDs();

}
