/**
 * Copyright (C) 2009  BonitaSoft S.A.
 * BonitaSoft, 31 rue Gustave Eiffel - 38000 Grenoble
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.services.impl;

import java.util.Set;

import org.ow2.bonita.facade.identity.impl.RoleImpl;
import org.ow2.bonita.facade.identity.impl.UserImpl;
import org.ow2.bonita.persistence.IdentityDbSession;
import org.ow2.bonita.persistence.db.HibernateDbSession;
import org.ow2.bonita.services.IdentityService;
import org.ow2.bonita.util.EnvTool;

/**
 * @author Anthony Birembaut
 *
 */
public class DbIdentity extends HibernateDbSession implements IdentityService {
  
  private String persistenceServiceName;

  public DbIdentity(String persistenceServiceName) {
    super();
    this.persistenceServiceName = persistenceServiceName;
  }

  protected IdentityDbSession getDbSession() {
    return EnvTool.getIdentityDbSession(persistenceServiceName);
  }
  
  public RoleImpl getRole(String name) {
    return getDbSession().getRole(name);
  }

  public Set<RoleImpl> getRoles() {
    return getDbSession().getRoles();
  }

  public UserImpl getUser(String username) {
    return getDbSession().getIdentityUser(username);
  }

  public Set<UserImpl> getUsers() {
    return getDbSession().getIdentityUsers();
  }

  public void addRole(RoleImpl role) {
    getDbSession().save(role);
  }

  public void addUser(UserImpl user) {
    getDbSession().save(user);
  }

  public void deleteRole(RoleImpl role) {
    getDbSession().delete(role);
  }

  public void deleteUser(UserImpl user) {
    getDbSession().delete(user);
  }
}
