/**
 * Copyright (C) 2010  BonitaSoft S.A.
 * BonitaSoft, 31 rue Gustave Eiffel - 38000 Grenoble
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.services.impl;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.ow2.bonita.facade.privilege.PrivilegePolicy;
import org.ow2.bonita.facade.privilege.Rule;
import org.ow2.bonita.facade.privilege.Rule.RuleType;
import org.ow2.bonita.facade.privilege.impl.RuleImpl;
import org.ow2.bonita.persistence.PrivilegeDbSession;
import org.ow2.bonita.persistence.db.HibernateDbSession;
import org.ow2.bonita.services.Journal;
import org.ow2.bonita.services.PrivilegeService;
import org.ow2.bonita.util.EnvTool;

/**
 * @author Nicolas Chabanoles
 * 
 */
public class DbPrivilegeService extends HibernateDbSession implements
    PrivilegeService {

  private static final String PRIVILEGE_DEFAULT_POLICY = "Privilege_default_policy";
  private String persistenceServiceName;

  public DbPrivilegeService(String persistenceServiceName) {
    super();
    this.persistenceServiceName = persistenceServiceName;
  }

  protected PrivilegeDbSession getDbSession() {
    return EnvTool.getPrivilegeDbSession(persistenceServiceName);
  }

  public void addRule(Rule rule) {
    getDbSession().save((RuleImpl) rule);
  }


  public Rule getRule(String name) {
    return getDbSession().getRule(name);
  }

  public Set<Rule> getRules() {
    return getDbSession().getRules();
  }


  public void deleteRule(RuleImpl rule) {
    getDbSession().delete(rule);
  }


  public void deleteRule(Rule rule) {
    getDbSession().delete(rule);

  }

  public Set<Rule> getRules(Set<String> rulesName) {
    return getDbSession().getRules(rulesName);
  }

  public PrivilegePolicy getDefaultPolicy() {
    Journal repo = EnvTool.getJournal();
    String defaultPolicy = repo.getMetaData(PRIVILEGE_DEFAULT_POLICY);
    if(defaultPolicy == null){
      return PrivilegePolicy.ALLOW_BY_DEFAULT;
    } else {
      try{
      return PrivilegePolicy.valueOf(defaultPolicy);
      }catch (Exception e) {
        e.printStackTrace();
        String allowedValues = ", ";
        for (PrivilegePolicy policy : PrivilegePolicy.values()) {
          allowedValues += policy.name();
        }
        allowedValues = allowedValues.substring(1);
        throw new IllegalArgumentException("The value of the privilege global policy is not valid: " + defaultPolicy + ". Only value defined in the PrivilegePolicy.class("+allowedValues+") are allowed.");
      }
    }
  }

  public Set<Rule> getRulesByType(RuleType... ruleTypes) {
    Set<String> ruleTypesStr = new HashSet<String>();
    for (RuleType ruleType : ruleTypes) {
      ruleTypesStr.add(ruleType.name());
    }
    return getDbSession().getRulesByType(ruleTypesStr);
  }

  public Set<Rule> getAllApplicableRules(String entityID) {
    return getDbSession().getAllApplicableRules(entityID);
  }

  public Set<Rule> getApplicableRules(String entityID, RuleType ruleType) {
    return getDbSession().getAllApplicableRules(entityID, ruleType);
  }

  public void setDefaultPolicy(PrivilegePolicy newPolicy) {
    Journal repo = EnvTool.getJournal();
    String defaultPolicy = repo.getMetaData(PRIVILEGE_DEFAULT_POLICY);
    
    if(newPolicy == null){
      // remove meta
      if(defaultPolicy !=null){
       repo.deleteMetaData(PRIVILEGE_DEFAULT_POLICY);
      }
    } else {
        repo.storeMetaData(PRIVILEGE_DEFAULT_POLICY, newPolicy.name());
    }
  }

  public List<Rule> getRules(RuleType ruleType, int fromIndex, int pageSige) {
    return getDbSession().getRules(ruleType, fromIndex, pageSige);
  }

  public Set<String> getExceptions(String entityID, RuleType ruleType) {
    return getDbSession().getAllExceptions(entityID, ruleType);
  }

  public long getNumberOfRules(RuleType ruleType) {
    return getDbSession().getNumberOfRules(ruleType);
  }

}
