/**
 * Copyright (C) 2009-2010  BonitaSoft S.A.
 * BonitaSoft, 31 rue Gustave Eiffel - 38000 Grenoble
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.services.impl;

import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.ow2.bonita.facade.ManagementAPI;
import org.ow2.bonita.facade.impl.StandardAPIAccessorImpl;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.persistence.DbSession;
import org.ow2.bonita.services.UUIDService;
import org.ow2.bonita.util.EnvTool;
import org.ow2.bonita.util.ProcessUtil;

/**
 * 
 * @author Charles Souillard
 *
 */
public class DbUUIDService implements UUIDService {

	static final Logger LOG = Logger.getLogger(ProcessUtil.class.getName());
	private static Map<ProcessDefinitionUUID, Long> uuids = new HashMap<ProcessDefinitionUUID, Long>();
	//private static Map<ActivityInstanceUUID, Long> iterationIds = new HashMap<ActivityInstanceUUID, Long>();
	
	private String persistenceServiceName;

	public DbUUIDService(final String persistenceServiceName) {
		this.persistenceServiceName = persistenceServiceName;
  }

	protected DbSession getDbSession() {
    return EnvTool.getQuerierDbSession(persistenceServiceName);
  }

	public synchronized ProcessInstanceUUID getNewProcessInstanceUUID(ProcessDefinitionUUID processUUID) {
		long processInstanceNb = 0;
		final String metaName = getMetadataName(processUUID);
		StandardAPIAccessorImpl accessor = new StandardAPIAccessorImpl();
		final ManagementAPI managementAPI = accessor.getManagementAPI();
		final String lastIntanceNb = managementAPI.getMetaData(metaName);
		if (lastIntanceNb != null) {
			processInstanceNb = Long.valueOf(lastIntanceNb);
			//managementAPI.deleteMetaData(metaName);
		}
    processInstanceNb++;
    processInstanceNb = getNextProcessInstanceNb(processInstanceNb, processUUID);
    managementAPI.addMetaData(metaName, Long.toString(processInstanceNb));
    if (LOG.isLoggable(Level.FINE)) {
      LOG.fine("Creating a new ProcessInstance with nb: " + processInstanceNb);
    }
		return new ProcessInstanceUUID(processUUID, processInstanceNb);
	}

	public synchronized void disableProcess(ProcessDefinitionUUID processUUID) {
	  final StandardAPIAccessorImpl accessor = new StandardAPIAccessorImpl();
	  final ManagementAPI managementAPI = accessor.getManagementAPI();
	  managementAPI.deleteMetaData(getMetadataName(processUUID));
	}
	
	
	private static synchronized long getNextProcessInstanceNb(final long processInstanceNb, ProcessDefinitionUUID processUUID) {
	  long processInstanceNumber = processInstanceNb;
	  if (uuids.containsKey(processUUID)) {
      long last = uuids.get(processUUID);
      if (last >= processInstanceNb) {
        processInstanceNumber = last + 1;
      }
    }
	  uuids.put(processUUID, processInstanceNumber);
	  return processInstanceNumber;
	}
	
	private String getMetadataName(final ProcessDefinitionUUID processUUID) {
    return "*****" + processUUID + "*****instance-nb*****";
  }
	
	/*
	public String getNewIterationId(ActivityInstanceUUID activityUUID) {
    long iterationId = 1;
    final String metaName = getMetadataName(activityUUID);
    final ManagementAPI managementAPI = AccessorUtil.getManagementAPI();
    final String lastIterationId = managementAPI.getMetaData(metaName);
    if (lastIterationId != null) {
      iterationId = Long.valueOf(lastIterationId);
      //managementAPI.deleteMetaData(metaName);
    }
    iterationId++;
    iterationId = getNextIterationId(iterationId, activityUUID);
    managementAPI.addMetaData(metaName, Long.toString(iterationId));
    if (LOG.isLoggable(Level.FINE)) {
      LOG.fine("Creating a new iteration id for activity " + activityUUID + " with nb: " + iterationId);
    }
    return "it" + iterationId;
  }
	
	private static synchronized long getNextIterationId(final long iterationId, ActivityInstanceUUID activityUUID) {
    long newIterationId = iterationId;
    if (iterationIds.containsKey(activityUUID)) {
      long last = iterationIds.get(activityUUID);
      if (last >= iterationId) {
        newIterationId = last + 1;
      }
    }
    iterationIds.put(activityUUID, newIterationId);
    return newIterationId;
  }
	
	private String getMetadataName(final ActivityInstanceUUID activityUUID) {
    return "*****" + activityUUID + "*****iteration-id*****";
  }
  */
}
