/**
 * Copyright (C) 2010  BonitaSoft S.A.
 * BonitaSoft, 31 rue Gustave Eiffel - 38000 Grenoble
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.type.lob;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.Serializable;
import java.sql.Blob;
import java.sql.SQLException;

public class BlobImpl implements Blob, Serializable {

  private static final long serialVersionUID = 3645219193341375020L;
  
  private InputStream stream;
  private int length;
  private boolean needsReset = false;

  public BlobImpl(byte[] bytes) {
    this.stream = new ByteArrayInputStream(bytes);
    this.length = bytes.length;
  }

  public BlobImpl(InputStream stream, int length) {
    this.stream = stream;
    this.length = length;
  }

  /**
   * @see java.sql.Blob#length()
   */
  public long length() throws SQLException {
    return length;
  }

  /**
   * @see java.sql.Blob#truncate(long)
   */
  public void truncate(long pos) throws SQLException {
    excep();
  }

  /**
   * @see java.sql.Blob#getBytes(long, int)
   */
  public byte[] getBytes(long pos, int len) throws SQLException {
    excep(); return null;
  }

  /**
   * @see java.sql.Blob#setBytes(long, byte[])
   */
  public int setBytes(long pos, byte[] bytes) throws SQLException {
    excep(); return 0;
  }

  /**
   * @see java.sql.Blob#setBytes(long, byte[], int, int)
   */
  public int setBytes(long pos, byte[] bytes, int i, int j)
  throws SQLException {
    excep(); return 0;
  }

  /**
   * @see java.sql.Blob#position(byte[], long)
   */
  public long position(byte[] bytes, long pos) throws SQLException {
    excep(); return 0;
  }

  /**
   * @see java.sql.Blob#getBinaryStream()
   */
  public InputStream getBinaryStream() throws SQLException {
    try {
      if (needsReset) stream.reset();
    }
    catch (IOException ioe) {
      throw new SQLException("could not reset reader");
    }
    needsReset = true;
    return stream;
  }

  public InputStream getBinaryStream(long pos, long length) throws SQLException {
    excep(); return null;
  }
  /**
   * @see java.sql.Blob#setBinaryStream(long)
   */
  public OutputStream setBinaryStream(long pos) throws SQLException {
    excep(); return null;
  }

  /**
   * @see java.sql.Blob#position(Blob, long)
   */
  public long position(Blob blob, long pos) throws SQLException {
    excep(); return 0;
  }

  private static void excep() {
    throw new UnsupportedOperationException("Blob may not be manipulated from creating session");
  }

}
