/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 * 
 * MOdified by Matthieu Chaffotte - BonitaSoft S.A.
 **/
package org.ow2.bonita.util;

import java.util.HashMap;
import java.util.Map;

import org.ow2.bonita.env.Environment;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.type.Converter;
import org.ow2.bonita.type.Type;
import org.ow2.bonita.type.Variable;
import org.ow2.bonita.type.VariableTypeResolver;
import org.ow2.bonita.type.variable.NullVariable;
import org.ow2.bonita.type.variable.UnpersistableVariable;

/**
 * @author Pierre Vigneras
 */
public abstract class VariableUtil {

  public static final String DEPLOYMENT_IDPREFIX = "deployment$";

  protected VariableUtil() { }

  public static Map<String, Variable> createVariableMap(final ProcessDefinitionUUID processUUID, final Map<String, Object> variables) {
    if (variables == null || variables.isEmpty()) {
      return null;
    }
    final Map<String, Variable> variableMap = new HashMap<String, Variable>();
    for (final Map.Entry<String, Object> e : variables.entrySet()) {
      final Object value = e.getValue();
      variableMap.put(e.getKey(), createVariable(processUUID, e.getKey(), value));
    }
    return variableMap;
  }

  /**
   * Copied from the pvm (in VariableMap)
   * TODO: factorize code
   */
  public static Variable createVariable(final ProcessDefinitionUUID processUUID, final String key, final Object value) {
    Type type = null;
    final Environment environment = Environment.getCurrent();
    if (environment != null) {
      final VariableTypeResolver variableTypeResolver = environment.get(VariableTypeResolver.class);
      if (variableTypeResolver != null) {
        type = variableTypeResolver.findTypeByMatch(key, value);
      }
    }
    Variable variable = null;
    if (type != null) {
      final Class< ? > variableClass = type.getVariableClass();
      try {
        variable = (Variable) variableClass.newInstance();
        variable.setProcessUUID(processUUID);
      } catch (final Exception e) {
      	String message = ExceptionManager.getInstance().getFullMessage("bafi_FCT_1", variableClass.getName());
        throw new BonitaRuntimeException(message);
      }
      final Converter converter = type.getConverter();
      variable.setConverter(converter);

    } else {
      if (value == null) {
        variable = new NullVariable();
      } else {
        variable = new UnpersistableVariable();
      }
    }

    variable.setKey(key);
    variable.setValue(value);
    return variable;
  }

  public static Map<String, Variable> copyVariableMap(final ProcessDefinitionUUID processUUID, Map<String, Variable> variables) {
	if (variables == null || variables.isEmpty()) {
	  return null;
	}
	final Map<String, Variable> variableMap = new HashMap<String, Variable>();
	for (final Map.Entry<String, Variable> e : variables.entrySet()) {
		Variable var = copyVariable(e.getValue());
	  variableMap.put(var.getKey(), var);
	}
	return variableMap;
  }
  
  public static Variable copyVariable(Variable var) {
	  final Object value = var.getValue();
	  final String key = var.getKey();
	  return createVariable(var.getProcessUUID(), key, value);
  }
}
